from django import forms
from django.conf import settings
from django.template import TemplateDoesNotExist
from django.template.loader import find_template_loader
from django.utils.translation import ugettext_lazy as _

from dbtemplates.admin import TemplateAdminForm as BaseTemplateForm, TemplateContentTextArea
from dbtemplates.models import Template

from player.base.forms import BaseForm


class TemplateAdminForm(BaseForm, BaseTemplateForm):
    content = forms.CharField(
        widget=TemplateContentTextArea({'rows': '18'}), required=False,
        help_text=_("Leaving this empty causes Django to look for a "
        "template with the given name and populate this field with its content."))

    def save(self, commit=True):
        tpl = super(TemplateAdminForm, self).save(commit)

        if not tpl.content:
            loaders = []
            for loader_name in settings.TEMPLATE_LOADERS:
                if loader_name == 'dbtemplates.loader.Loader':
                    continue  # we miss dbtemplates
                loader = find_template_loader(loader_name)
                if loader is not None:
                    loaders.append(loader)
            for loader in loaders:
                try:
                    tpl_source, file_path = loader.load_template_source(tpl.name)
                    tpl.content = tpl_source
                    break
                except TemplateDoesNotExist:
                    pass  # continue to next loader
        if commit:
            tpl.save()
        return tpl

    class Meta:
        model = Template
        exclude = ('sites', 'creation_date', 'last_changed', )



from django.http import HttpResponse


class RenderBlockMiddleware(object):
    """This middleware render only a block if passed by request """

    def process_request(self, request):
        if 'render_block' in request.GET:
            from player.block.models import PlacedBlock

            block_id = request.GET['render_block']
            placed_block = PlacedBlock.objects.get(id=block_id)
            block = placed_block.get_block()
            return HttpResponse(block.render(
                request,
                context=dict(),
            ))



from django.contrib import admin


class ModelAdmin(admin.ModelAdmin):
    list_display = ('id', '__unicode__', 'is_deleted')
    list_display_filter = ('is_deleted', )

    def queryset(self, request):
        qs = self.model._default_manager.with_deleted()
        ordering = self.ordering or ()
        if ordering:
            qs = qs.order_by(*ordering)
        return qs



from django.contrib import admin

from player.crawler.models import CrawlerCollection, Report


class CrawlerCollectionAdmin(admin.ModelAdmin):
    change_form_template = 'admin/crawler/collection_change_form.html'


admin.site.register(CrawlerCollection, CrawlerCollectionAdmin)
admin.site.register(Report)



from datetime import datetime
from django.db import models
from django.db.models import signals
from django.utils.translation import ugettext_lazy as _


class LogicalDeletedManager(models.Manager):

    def get_query_set(self):
        if self.model:
            return super(LogicalDeletedManager, self).get_query_set().filter(is_deleted=False)

    def with_deleted(self):
        if self.model:
            return super(LogicalDeletedManager, self).get_query_set()

    def only_deleted(self):
        if self.model:
            return super(LogicalDeletedManager, self).get_query_set().filter(is_deleted=True)

    def get(self, *args, **kwargs):
        ''' if a specific record was requested, return it even if it's deleted '''
        return self.with_deleted().get(*args, **kwargs)

    def filter(self, *args, **kwargs):
        ''' if pk was specified as a kwarg, return even if it's deleted '''
        if 'pk' in kwargs or 'id' in kwargs:
            return self.with_deleted().filter(*args, **kwargs)
        return self.get_query_set().filter(*args, **kwargs)


class Model(models.Model):
    date_deleted = models.DateTimeField(null=True, blank=True)
    is_deleted = models.BooleanField(_('is deleted'), default=False)

    objects = LogicalDeletedManager()

    def delete(self, using=None, purge=False):
        if not purge:
            signals.pre_delete.send(sender=self.__class__, instance=self)
            self.date_deleted = datetime.now()
            self.is_deleted = True
            self.save(using=using)
        else:  # we remove object
            super(Model, self).delete(using=using)

    class Meta:
        abstract = True



import datetime
import hashlib
import feedparser

from django.conf import settings
from django.utils.encoding import smart_str

from player.crawler.models import CrawlerCollection
from player.crawler.backends.utils import collect_items_extracted, create_extraction_report
from player.data.models import Collection, Item


def get_collection_link(crawler_collection):
    """
    This function will return a representative link to collection configured with
    the scrap tool.
    """
    return crawler_collection.collection_code


def get_collection_schema(collection_code):
    """
    This function will return the collection schema configured in your scrap tool
    with collection_code
    """
    feed = feedparser.parse(collection_code)
    first_item = feed.entries[0]
    schema = {}
    for field_name in first_item.keys():
        schema[field_name] = {u'id': field_name,
                              u'type': 'text',
                              u'is_match_up': False}
    # some replacements because RSS specs
    schema['link'].update({'type': 'url', 'is_match_up': True})
    if 'content' in schema:
        schema['content'].update({'type': 'html'})
    return schema


def load_collection_schema(crawler_collection):
    """
    This function will load the collection schema configured in your scrap tool
    with collection_code
    """
    schema = get_collection_schema(crawler_collection.collection_code)
    if schema:
        crawler_collection.schema = schema
        crawler_collection.save()


def has_new_data(crawler_collection):
    """ return True if scraping tool have new data to extract """
    # TODO: to implement
    return True


def extract_collection_data(crawler_collection):
    """
    This function will parse the collection as configured in your scrap tool.
    """
    collection_code = crawler_collection.collection_code
    collections_to_extract = Collection.objects.filter(crawler_collection__collection_code=collection_code)
    items_dict = {'added': [], 'updated': [], 'deleted': []}
    for collection in collections_to_extract:
        items_extracted = []
        has_matchup_fields = collection.fields.is_mapped().filter(is_matchup=True).exists()
        feed = feedparser.parse(collection_code)
        for entry in feed.entries:
            uid = hashlib.md5()
            field_dict = {}
            for field in collection.fields.all().is_mapped():
                field_name = field.mapped_field
                value = entry[field_name]
                if field.is_matchup or not has_matchup_fields:
                    uid.update(smart_str(value))
                field_dict[field.slug] = value
                item = Item(uid=uid.hexdigest(), collection=collection)
                item.content = field_dict
                items_extracted.append(item)
        success, log, added_items, updated_items, deleted_items_uid, deleted_items = collect_items_extracted(items_extracted, collection)
        collection.last_crawling_time = datetime.datetime.now()
        collection.save()
        items_dict['added'] += added_items
        items_dict['updated'] += updated_items
        items_dict['deleted'] += deleted_items
        crawler_collection.last_crawling_time = datetime.datetime.now()
        crawler_collection.last_job_time = datetime.datetime.now()
        crawler_collection.save()
        affected_collections = Collection.objects.filter(crawler_collection=crawler_collection)

        create_extraction_report(success, crawler_collection, items_dict, log)


def get_collection_codes(login, password=None):
    """ returns None because code is edited by hand"""
    return None


# ----- testing functions (used in unit tests) -----


def create_test_crawler_collection():
    """ Create test collection """
    return CrawlerCollection.objects.create(
            collection_code=settings.CRAWLER_TEST_COLLECTION_CODE,
    )


def run_test_extraction_process():
    """ Run extraction of test data in scraping tool """
    raise NotImplementedError()



from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _

from player.data.models import Item


def item_list(request):
    items = Item.objects.all()
    return render_to_response('items/item_list.html',
                              {'item_list': items},
                              context_instance=RequestContext(request))


def item_detail(request, item_id):
    item = get_object_or_404(Item, pk=item_id)
    return render_to_response('items/item_detail.html',
                              {'item': item},
                              context_instance=RequestContext(request))


def views_to_register():
    """ returns views to be registered with dbresolver """
    return (
        (item_list, _('Item listing from all collections')),
        (item_detail, _('Item detail')),
    )



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.contrib.contenttypes.models import ContentType
from django.http import HttpResponseRedirect, Http404
from django.template import RequestContext
from django.shortcuts import render_to_response
from django.utils.translation import ugettext as _

from player.crawler.models import Report
from player.data.models import Collection


@login_required
def index(request):
    collections_draft = Collection.objects.all().draft()
    reports_error = Report.objects.all().with_errors()
    return render_to_response(
        "manage/index.html",
        {'collection_list': collections_draft,
         'reports': reports_error},
        context_instance=RequestContext(request),
    )


@login_required
def generic_delete(request):
    content_type_id = request.POST.get('content_type_id', None)
    content_id = request.POST.get('content_id', None)
    if content_type_id is None or content_id is None:
        raise Http404

    content_type = ContentType.objects.get(id=int(content_type_id))
    if request.user.has_perm('delete_%s' % content_type.model) or \
       request.user.has_perm('%s.delete_%s' % (content_type.app_label, content_type.model)):
        obj = content_type.get_object_for_this_type(id=content_id)
        obj.delete()
        messages.success(request, _('%(content)s deleted successfully') % {'content': obj})
    else:
        messages.error(request, _('You do not have permission to delete %(model)s objects') % {'model': content_type.model})
    next_url = request.POST.get('next_url', request.META.get('HTTP_REFERER', '/'))
    return HttpResponseRedirect(next_url)


@login_required
def content_delete(request):
    collection_id = request.POST.get('collection_id', None)
    if not collection_id:
        collection_id = request.POST.get('content_id', None)

    if collection_id is None:
        raise Http404

    if request.user.has_perm('delete_item') or \
       request.user.has_perm('data.delete_item'):
        collection = Collection.objects.get(id=collection_id)
        for item in collection.item_set.with_invalids():
            item.delete()
        messages.success(request, _('Items removed properly'))
    else:
        messages.error(request, _('You do not have permission to delete items objects'))
    next_url = request.POST.get('next_url', request.META.get('HTTP_REFERER', '/'))
    return HttpResponseRedirect(next_url)



# -*- coding: utf-8 -*-
"""
Crawler tests. This test only the scraping tool and extracted data.
"""
import time
import socket
import SocketServer
import threading
import simplejson as json
from datetime import date
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer
from lxml.etree import XMLSyntaxError

# from django.conf import settings
from django.test import TestCase
from django.contrib.sites.models import Site
from django.utils.datastructures import SortedDict
from django.utils.html import strip_tags

from player.crawler import get_backend
from player.crawler.models import CrawlerCollection
from player.crawler.exceptions import CrawlerHttpException
from player.data.models import Collection, CollectionField, Item
from testwebsite.models import NewsItem

TEST_BACKEND = 'player.crawler.backends.mozenda'

MAPPED_FIELDS = SortedDict({
    'content-title': {
        'name': 'Content title',
        'type': 'text',
        'mapped_field': 'name',
        'is_matchup': True,
    },
    'content-description': {
        'name': 'Content description',
        'type': 'html',
        'mapped_field': 'short description',
        'is_matchup': False,
    },
    'content-body': {
        'name': 'Content body',
        'type': 'html',
        'mapped_field': 'long description',
        'is_matchup': False,
    },
})

# Note: previous mapping only works with a Collection defined as follows:
# Fields:
#  - "link": a link to the content.
#  - "short description": content description.
#  - "long description": content body.
#  - "name": the content title

RESPONSE_COLLECTION_GET_FIELDS = """
"""

RESPONSE_COLLECTION_GET_VIEWS = """
"""

RESPONSE_VIEW_GET_ITEMS = """
"""

RESPONSE_AGENT_GET_JOBS = """
"""


class FakeServer(SocketServer.ThreadingTCPServer):
    allow_reuse_address = 1    # Seems to make sense in testing environment

    def server_bind(self):
        """Override server_bind to store the server name."""
        SocketServer.TCPServer.server_bind(self)
        host, port = self.socket.getsockname()[:2]
        self.server_name = socket.getfqdn(host)
        self.server_port = port


class FakeHandler(BaseHTTPRequestHandler):

    def get_datas(self, path, band):
        if band == 0:
            if 'Collection.GetViews' in path:
                datas = RESPONSE_COLLECTION_GET_VIEWS
            elif 'View.GetItems' in path:
                datas = RESPONSE_VIEW_GET_ITEMS
            elif 'Collection.GetFields' in path:
                datas = RESPONSE_COLLECTION_GET_FIELDS
            elif 'Agent.GetJobs' in path:
                datas = RESPONSE_AGENT_GET_JOBS
        else:
            datas = ''
        return datas

    def do_GET(self):
        if self.server.band <= 1:
            self.send_response(200)
        elif self.server.band == 2:
            self.send_response(500)
        elif self.server.band == 3:
            self.send_response(400)
        datas = self.get_datas(self.path, self.server.band)
        self.send_header('Content-type', 'text/xml; charset=utf-8')
        self.server.requests_processed.append(
            {'method': 'GET', 'path': self.path, 'data': datas})
        self.end_headers()
        self.wfile.write(datas)


class MyThread(threading.Thread):

    def __init__(self, band, number_of_requests=1, port=8000):
        super(MyThread, self).__init__()
        self.number_of_requests = number_of_requests
        self.requests_processed = []
        self.band = band
        self.port = port

    def run(self):
        fake_server = HTTPServer(('', self.port), FakeHandler)
        fake_server.requests_processed = []
        fake_server.band = self.band
        while self.number_of_requests > 0:
            fake_server.handle_request()
            self.number_of_requests -= 1
            self.requests_processed = fake_server.requests_processed
            self.band = fake_server.band
        fake_server.socket.close()


class CrawlerTest(TestCase):

    def setUp(self):
        self.backend = get_backend(TEST_BACKEND)
        self.site = Site.objects.get_current()
        self.crawler_collection = self.backend.create_test_crawler_collection()
        # uncomment next line if you want to run extraction jobs in test time (very very slow)
        #self.backend.run_test_extraction_process()
        self.collection = Collection.objects.create(
            name='test collection',
            crawler_collection=self.crawler_collection,
        )
        for field_slug, mapping in MAPPED_FIELDS.items():
            CollectionField.objects.create(
                name=mapping['name'],
                slug=field_slug,
                field_type=mapping['type'],
                mapped_field=mapping['mapped_field'],
                is_matchup=mapping['is_matchup'],
                collection=self.collection,
            )
        self.backend.extract_collection_data(self.crawler_collection)

    def test_crawled_data_schema(self):
        """
        Tests crawled data schema is correct
        """
        crawled_items = self.collection.item_set.all()
        for item in crawled_items:
            for field_name, value in item.content.items():
                # field_name must be in MAPPED_FIELDS keys (field slugs)
                self.failIf(field_name not in MAPPED_FIELDS)
                # do type checking
                mapping_type = MAPPED_FIELDS[field_name]['type']
                field_value = item.get_value(field_name)
                if mapping_type in ('text', 'html', 'url'):
                    self.failUnless(isinstance(field_value, unicode))
                elif mapping_type == 'date':
                    self.failUnless(isinstance(field_value, date))
                elif mapping_type == 'int':
                    self.failUnless(isinstance(field_value, int))

    def test_crawled_data(self):
        """
        Tests crawled data is correct
        """
        crawled_items = self.collection.item_set.all()
        collection_items = self.collection.item_set.values()
        newsitem_list = NewsItem.objects.all()

        for item_extract in crawled_items:
            for it in collection_items:
                if it['id'] == item_extract.id:
                    item_database = json.loads(it['content'])
            for it in newsitem_list:
                if it.title == item_extract.content['content-title']:
                    item_newitem = it

            for field_name, value in item_extract.content.items():
                self.failIf(field_name not in MAPPED_FIELDS)
                field_value = item_extract.get_value(field_name)
                self.assertEquals(field_value, item_database[field_name])
                if 'body' in field_name:
                    self.assertEquals(strip_tags(item_newitem.body),
                                      strip_tags(field_value))
                elif 'title' in field_name:
                    self.assertEquals(strip_tags(item_newitem.title),
                                      strip_tags(field_value))
                elif 'description' in field_name:
                    self.assertEquals(strip_tags(item_newitem.description),
                                      strip_tags(field_value))


def run_server(band, num_request):
        time.sleep(1)
        server = MyThread(band, num_request, 8000)
        server.start()
        time.sleep(2)

        return server


class ExtractDataTest(TestCase):

    def _fixture_setup(self):
        # we need to run the harvester server here because
        # the fixtures creates objects and the post-save
        # signals tries to connec to the harvester
        server = run_server(0, 4)
        super(ExtractDataTest, self)._fixture_setup()
        server.number_of_requests = 0
        server.join()

    def setUp(self):
        self.assertEquals(Item.objects.count(), 0)
        self.backend = get_backend(TEST_BACKEND)
        # self.site = Site.objects.get_current()
        self.crawler_collection = CrawlerCollection.objects.get_or_create(collection_code='1014')[0]

        # server = run_server(0, 1)
        # self.crawler_collection = self.backend.create_test_crawler_collection()
        # uncomment next line if you want to run extraction jobs in test time (very very slow)
        #self.backend.run_test_extraction_process()
        self.collection = Collection.objects.get_or_create(crawler_collection=self.crawler_collection)[0]
        # server.number_of_requests = 0
        # server.join()

    def test_extract_data_ok(self):
        server = run_server(0, 3)
        self.backend.extract_collection_data(self.crawler_collection)
        server.number_of_requests = 0
        server.join()
        # comprobar todos los items creados
        self.assertEqual(len(Item.objects.all()), 2)

    def test_extract_data_without_xml(self):
        server = run_server(1, 1)
        self.assertRaises(XMLSyntaxError,
                          self.backend.extract_collection_data,
                          (self.crawler_collection))
        # recibe una respuesta 200 con content-type: text/xml pero sin datos.
        server.number_of_requests = 0
        server.join()
        self.assertEqual(len(Item.objects.all()), 0)

    def test_extract_data_server_error(self):
        server = run_server(2, 1)
        self.assertRaises(CrawlerHttpException,
                          self.backend.extract_collection_data,
                          (self.crawler_collection))
        # server error (500)
        server.number_of_requests = 0
        server.join()

    def test_extract_data_client_error(self):
        server = run_server(3, 1)
        self.assertRaises(CrawlerHttpException,
                          self.backend.extract_collection_data,
                          (self.crawler_collection))
        # server error (400)
        server.number_of_requests = 0
        server.join()



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-player.
#
# django-player is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-player.  If not, see <http://www.gnu.org/licenses/>.


def collections(request):
    from player.data.models import Collection
    collections = {}
    for collection in Collection.objects.all():
        collections[collection.slug] = collection.item_set.all()
    return {'collections': collections}



from django.db import models
from django.db.models.signals import post_save
from django.utils.translation import ugettext_lazy as _

from configfield.dbfields import JSONField
from player.crawler import get_backend, get_backends
from player.crawler.managers import ReportManager


REPORT_RETURN_CODES = (
    ('unk', _('Unknow')),
    ('suc', _('Success')),
    ('err', _('Error')),
    ('net', _('New elements treshold exceeded')),
    ('met', _('Modified elements treshold exceeded')),
    ('det', _('Deleted elements treshold exceeded')),
)


class Report(models.Model):
    crawler_collection = models.ForeignKey('CrawlerCollection')
    crawling_time = models.DateTimeField(
        _('Crawling time'),
        null=True,
        blank=True,
    )
    return_code = models.CharField(
        _('Return code'),
        max_length=3,
        choices=REPORT_RETURN_CODES,
    )
    traceback = models.TextField(
        _('Traceback'),
        blank=True,
        null=True,
    )
    crawled_data = models.TextField(
        _('Crawled data'),
    )
    added_items = models.ManyToManyField(
        'data.Item',
        blank=True,
        null=True,
        related_name='added_in_reports',
    )
    updated_items = models.ManyToManyField(
        'data.Item',
        blank=True,
        null=True,
        related_name='updated_in_reports',
    )
    deleted_items_json = JSONField(
        blank=True,
        null=True,
    )

    objects = ReportManager()

    class Meta:
        get_latest_by = 'crawling_time'
        ordering = ('-crawling_time', )

    def __unicode__(self):
        return u'%s (%s)' % (self.crawler_collection, self.crawling_time.strftime('%d/%m/%Y %H:%M:%S'))

    @models.permalink
    def get_absolute_url(self):
        return ('report_view', (self.id, ))


class CrawlerCollection(models.Model):
    """ Represent a scraping tool content type (news, events, etc) """
    backend = models.CharField(max_length=200,
                               choices=get_backends(),
                               help_text=_('Unique code for integrating with extraction tool collection'))
    collection_code = models.CharField(max_length=200,
                                       help_text=_('Unique code for integrating with extraction tool collection'))
    schema = JSONField(editable=False)
    last_crawling_time = models.DateTimeField(_('Last crawling time'),
                                              null=True, blank=True)
    last_job_time = models.DateTimeField(_('Last job made in extraction tool'),
                                         null=True, blank=True)

    check_deleted = models.BooleanField(_('Check deleted'), default=False)
    # this threshold values are CharField instead of Integer to allow percentages
    new_elements_threshold = models.CharField(_('Maximum number of new elements'),
        max_length=200, null=True, blank=True)
    modified_elements_threshold = models.CharField(_('Maximum number of modified elements'),
        max_length=200, null=True, blank=True)
    deleted_elements_threshold = models.CharField(_('Maximum number of deleted elements'),
        max_length=200, null=True, blank=True)

    class Meta:
        verbose_name = _('Crawler collection')

    def __unicode__(self):
        return self.collection_code


def handle_collection_save(sender, instance, created, **kwargs):
    if created:
        get_backend(instance.backend).load_collection_schema(instance)

post_save.connect(handle_collection_save, sender=CrawlerCollection)



from django.conf import settings
from django.conf.urls.defaults import patterns, url


slug_re = settings.SLUG_RE

urlpatterns = patterns('player.data.views',
    (r'^collectiongroups/$', 'collectiongroup_list'),
    url(r'^collectiongroups/add/$', 'collectiongroup_add', name='collectiongroup_add'),
    url(r'^collectiongroups/(?P<slug>%s)/$' % slug_re, 'collectiongroup_view', name='collectiongroup_view'),
    url(r'^collectiongroups/(?P<slug>%s)/edit/$', 'collectiongroup_edit', name='collectiongroup_edit'),
    url(r'^collectiongroups/(?P<slug>%s)/collection/(?P<collection_slug>%s)/$' % (slug_re, slug_re), 'collection_view', name='collection_view'),
    url(r'^collectiongroups/(?P<slug>%s)/collection/(?P<collection_slug>%s)/edit/$' % (slug_re, slug_re), 'collection_edit', name='collection_edit'),
    url(r'^collectiongroups/(?P<slug>%s)/collection/(?P<collection_slug>%s)/scrap_association/$' % (slug_re, slug_re),
        'scrap_association', name='scrap_association'),
    (r'^collectiongroups/(?P<slug>%s)/collection/(?P<collection_slug>%s)/publish/$' % (slug_re, slug_re), 'collection_publish'),
    (r'^collectiongroups/(?P<slug>%s)/collection/(?P<collection_slug>%s)/unpublish/$' % (slug_re, slug_re), 'collection_unpublish'),
    url(r'^collectiongroups/(?P<slug>%s)/collection/(?P<collection_slug>%s)/crawl/$' % (slug_re, slug_re), 'collection_crawl', name='collection_crawl'),
    (r'^collectiongroups/(?P<slug>%s)/collection/(?P<collection_slug>%s)/edit-fields/$' % (slug_re, slug_re), 'collection_edit_fields'),
    (r'^collectiongroups/(?P<slug>%s)/collection/(?P<collection_slug>%s)/fields-in-json/$' % (slug_re, slug_re), 'collection_fields_in_json'),
    url(r'^collectiongroups/(?P<slug>%s)/collection/(?P<collection_slug>%s)/steps/toggle-crawling-checked$' % (slug_re, slug_re),
        'collection_toggle_crawling_checked', name='collection_toggle_crawling_checked'),
    (r'^collectiongroups/(?P<slug>%s)/item/(?P<item_id>\w+)/$' % settings.SLUG_RE, 'item_view'),
    (r'^collectiongroups/(?P<slug>%s)/item/(?P<item_id>\w+)/validate/$' % settings.SLUG_RE, 'item_validate'),
    (r'^collections/$', 'collection_list'),
    url(r'^collections/add/$', 'collection_add', name='collection_add'),
)



# Copyright (c) 2010 by Manuel Saelices
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.utils.translation import ugettext_lazy as _


class NewsCategory(models.Model):
    title = models.CharField(max_length=200)
    slug = models.SlugField(max_length=100)

    class Meta:
        verbose_name = _('news category')
        verbose_name_plural = _('news categories')

    def __unicode__(self):
        return self.title


class NewsItem(models.Model):
    title = models.CharField(max_length=200)
    slug = models.SlugField(max_length=100)

    publish_date = models.DateTimeField(db_index=True)
    category = models.ForeignKey(NewsCategory,
                                 verbose_name=_('category'))
    body = models.TextField(_('body'))

    class Meta:
        ordering = ('-publish_date', '-id')
        verbose_name = _('news item')
        verbose_name_plural = _('news')

    def __unicode__(self):
        return self.title

    @models.permalink
    def get_absolute_url(self):
        return ('news_detail', (), {'news_slug': self.slug})



# Copyright (c) 2010 by Manuel Saelices
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django.db import models
from django.utils.translation import ugettext_lazy as _

from configfield.dbfields import ConfigField
from player.block import get_block as get_block_by_path
from player.block.managers import PlacedBlockManager


class PlacedBlock(models.Model):
    name = models.CharField(max_length=200)
    block_path = models.CharField(max_length=200)
    placed_at = models.CharField(max_length=100, db_index=True)
    order = models.PositiveIntegerField(db_index=True)
    is_active = models.BooleanField(default=True)
    config = ConfigField(_('Configuration'), null=True, blank=True)

    objects = PlacedBlockManager()

    class Meta:
        ordering = ('order', )

    def __unicode__(self):
        return self.name

    def get_block(self):
        if hasattr(self, '_block'):
            return self._block  # cached block
        self._block = get_block_by_path(self.block_path, self)
        return self._block

    def show(self, place, request, context):
        return self.get_block().show(place, request, context)

    def render(self, request, context):
        return self.get_block().render(request, context)



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from django.utils.importlib import import_module


def get_backend(backend_mod_name):
    """ Returns crawler backend """
    try:
        backend = import_module(backend_mod_name)
    except ImportError, e:
        raise ImproperlyConfigured(('Error importing crawler backend module %s: "%s"'
                                    % (backend_mod_name, e)))
    return backend


def get_backends():
    """ Returns all crawler backends """
    return settings.CRAWLER_BACKENDS



from django.contrib import admin

from news.models import NewsItem, NewsCategory


class NewsAdmin(admin.ModelAdmin):
    prepopulated_fields = {"slug": ("title",)}

admin.site.register(NewsCategory, NewsAdmin)
admin.site.register(NewsItem, NewsAdmin)



from django.utils.translation import ugettext_lazy as _

from player.block import Block, register_block


class NavBlock(Block):
    """ Navigation block """
    name = 'navigationblock'
    label = _('Navigation block')

    def render(self, request, context):
        return self.render_block(request, template_name='website/nav_block.html',
                          block_title="Nav block",
                          context=context)

register_block(NavBlock)



from django.core.management.base import BaseCommand

from player.crawler import get_backend
from player.data.models import Collection


class Command(BaseCommand):

    option_list = BaseCommand.option_list + ()
    help = "Launch crawling in a all collections"
    requires_model_validation = False
    can_import_settings = True

    def handle(self, *args, **options):
        for collection in Collection.objects.all():
            crawler_collection = collection.crawler_collection
            print '   Analyzing "%s" collection...' % collection
            backend = get_backend(crawler_collection.backend)
            if backend.has_new_data(crawler_collection):
                print '\tCrawling "%s" collection...' % collection
                backend.extract_collection_data(crawler_collection)
                print '\tData extracted'
            else:
                print '\tCollection "%s" was up to date' % collection



from django.db.models import Manager
from player.logicaldelete.models import LogicalDeletedManager
from django.db.models.query import QuerySet


class ItemQuerySet(QuerySet):

    def contents(self):
        for item in self:
            yield item.content


class ItemManager(LogicalDeletedManager):
    use_for_related_fields = True

    def all(self):
        return self.get_query_set().filter(is_valid=True)

    def with_invalids(self):
        return self.get_query_set()

    def invalids(self):
        return self.get_query_set().filter(is_valid=False)

    def contents(self):
        return self.get_query_set().contents()

    def get_query_set(self):
        """ by default we will do joins with Collection for reduce SQL sentences """
        return ItemQuerySet(self.model).filter(is_deleted=False).select_related('collection')


class CollectionManager(Manager):

    def get_query_set(self):
        return CollectionQuerySet(self.model)


class CollectionQuerySet(QuerySet):

    def by_status(self, status):
        return self.filter(status=status)

    def published(self):
        """ only published objects """
        return self.by_status('published')

    def draft(self):
        """ only draft objects """
        return self.by_status('draft')


class CollectionFieldQuerySet(QuerySet):

    def is_mapped(self):
        return self.filter(mapped_field__isnull=False)


class CollectionFieldManager(Manager):
    use_for_related_fields = True

    def is_mapped(self):
        return self.get_query_set().is_mapped()

    def get_query_set(self):
        return CollectionFieldQuerySet(self.model)



# Copyright (c) 2010 by Manuel Saelices
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _

from dbresolver.forms import URLPatternForm
from dbresolver.models import URLPattern


@login_required
def url_list(request):
    """ List URLs patterns stored in database """
    urlpatterns = URLPattern.objects.all()
    return render_to_response('dburl/url_list.html', {'url_list': urlpatterns},
                              context_instance=RequestContext(request))


@login_required
def url_detail(request, url_id):
    """ View a URL pattern stored in database """
    url = URLPattern.objects.get(pk=url_id)
    return render_to_response('dburl/url_detail.html', {'url': url},
                              context_instance=RequestContext(request))


@login_required
def url_add(request, url=None):
    """ Add a new URL pattern to store in database """
    if request.method == 'POST':
        form = URLPatternForm(request.POST, instance=url)
        if form.is_valid():
            url = form.save()
            msg = url and _(u'The URL was successfully modified') or \
                  _(u'The URL was successfully added')
            messages.success(request, msg)
            return HttpResponseRedirect(reverse('url_detail', args=(url.id, )))
    else:
        form = URLPatternForm(instance=url)
    return render_to_response(
        "dburl/url_edit.html",
        {'form': form, 'url': url},
        context_instance=RequestContext(request),
    )


@login_required
def url_edit(request, url_id):
    """ Edit a URL pattern to store in database """
    url = get_object_or_404(URLPattern, pk=url_id)
    return url_add(request, url)



from django.contrib import admin

from dbtemplates.admin import TemplateAdmin as BaseTemplateAdmin
from dbtemplates.models import Template

from player.dbtemplate.forms import TemplateAdminForm

# deregistering legacy Template model admins
admin.site.unregister(Template)


class TemplateAdmin(BaseTemplateAdmin):
    form = TemplateAdminForm
    fieldsets = (
        (None, {
            'fields': ('name', 'content'),
            'classes': ('monospace',),
        }),
    )


admin.site.register(Template, TemplateAdmin)



from BeautifulSoup import BeautifulSoup


def is_included(node, media_list):
    if node.name == 'script':
        attrs_to_compare = ('src', 'id', )
    else:  # css
        attrs_to_compare = ('href', 'id', )
    for media in media_list:
        if node.name != media.name:
            continue
        for attr in attrs_to_compare:
            if node.get(attr) != media.get(attr):
                break
            # all attributes are equal
            return True
    return False


class MediaList(list):

    def __init__(self, namespace):
        self._namespace = namespace
        super(MediaList, self).__init__()

    def append(self, obj):
        if obj not in self:
            super(MediaList, self).append(obj)

    def render(self):
        soup = BeautifulSoup('\n'.join(self))
        media_assets = []
        for node in soup.findAll():  # remove duplicated media assets
            if not is_included(node, media_assets):
                media_assets.append(node)
        return '\n'.join([unicode(node) for node in media_assets])


class MediaDictionary(dict):
    """
    A media dictionary which auto fills itself instead of raising key errors.
    """

    def __init__(self):
        super(MediaDictionary, self).__init__()

    def __getitem__(self, item):
        if item not in self:
            self[item] = MediaList(item)
        return super(MediaDictionary, self).__getitem__(item)



from django import template
from django.utils.translation import ugettext as _

register = template.Library()


@register.inclusion_tag('data/get_collection_status.html',
                        takes_context=True)
def get_collection_status(context, collection):
    img_base = '%smanage/img/' % context['STATIC_URL']
    if collection.is_publishable():
        alt = _('Yes')
        img = 'step-done_16.png'
    else:
        alt = _('No')
        img = 'step-undone_16.png'
    img_path = '%s%s' % (img_base, img)
    return {'path': img_path, 'alt': alt}



from django.db import models
from django.db.models.query import QuerySet


class BlockQuerySet(QuerySet):

    def actives(self):
        return self.filter(is_active=True)


class PlacedBlockManager(models.Manager):
    """ Block manager """

    def get_query_set(self):
        return BlockQuerySet(self.model)

    def actives(self):
        return self.get_query_set().actives()



#!/usr/bin/env python

# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-player.
#
# django-player is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-player.  If not, see <http://www.gnu.org/licenses/>.

import os
import stat
import re
from random import choice
from optparse import OptionParser

from django.core.management.base import CommandError
from django.core.management.color import color_style
from django.utils.importlib import import_module

from player.base.management.base import copy_helper


def create_website(project_name, verbose=False):
    # Determine the project_name a bit naively -- by looking at the name of
    # the parent directory.
    directory = os.getcwd()

    # Check that the project_name cannot be imported.
    try:
        import_module(project_name)
    except ImportError:
        pass
    else:
        raise CommandError("%r conflicts with the name of an existing Python module and cannot be used as a project name. Please try another name." % project_name)

    copy_helper(color_style(), project_name, directory)

    # Create a random SECRET_KEY hash, and put it in the main settings.
    main_settings_file = os.path.join(directory, project_name, 'settings.py')
    settings_contents = open(main_settings_file, 'r').read()
    fp = open(main_settings_file, 'w')
    secret_key = ''.join([choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])
    settings_contents = re.sub(r"(?<=SECRET_KEY = ')'", secret_key + "'", settings_contents)
    fp.write(settings_contents)
    fp.close()
    # set manage.py as an executable file
    manage_file_path = os.path.join(directory, project_name, 'manage.py')
    mode = os.stat(manage_file_path)[stat.ST_MODE]
    os.chmod(manage_file_path, mode | stat.S_IXUSR)


def main():
    usage = "usage: %prog [options] project_name"
    parser = OptionParser(usage)
    parser.add_option("-q", "--quiet",
                  action="store_false", dest="verbose", default=True,
                  help="don't print status messages to stdout")
    (options, args) = parser.parse_args()
    if len(args) != 1:
        parser.error("You need to define the project name to create")
    create_website(args[0], options.verbose)


if __name__ == "__main__":
    main()



from django.db.models import Manager
from django.db.models.query import QuerySet


class ReportQuerySet(QuerySet):

    def by_return_code(self, code):
        return self.filter(return_code=code)

    def except_return_code(self, code):
        return self.exclude(return_code=code)

    def success(self):
        """ only success objects """
        return self.by_return_code('suc')

    def with_errors(self):
        """ exclude success objects """
        return self.except_return_code('suc')


class ReportManager(Manager):

    def get_query_set(self):
        return ReportQuerySet(self.model)



class CrawlerHttpException(Exception):

    def __init__(self, http_code, error):
        self.http_code = http_code
        self.error = error

    def __str__(self):
        return "[%s] %s" % (self.http_code, self.error)



from django.http import Http404
from django.shortcuts import render_to_response
from django.template import TemplateDoesNotExist, RequestContext
from django.utils.translation import ugettext as _
from django.views.generic.simple import direct_to_template

from configfield import params
from player.data.params import CollectionParam


DEFAULT_ITEMS_NUMBER = 20
DEFAULT_COLLECTION_TEMPLATE = 'item_list.html'
DEFAULT_ITEM_TEMPLATE = 'item_detail.html'


def render_template(request, template_name=None):
    try:
        return direct_to_template(request, template=template_name)
    except TemplateDoesNotExist:
        raise Http404()


def collection_template(request, template_name=DEFAULT_COLLECTION_TEMPLATE, collection_slug=None, items_number=DEFAULT_ITEMS_NUMBER):
    from player.data.models import Collection
    if collection_slug is None:
        raise Http404()
    collection = Collection.objects.get(slug=collection_slug)
    items = collection.item_set.all()
    if items_number != -1:
        items = items[:items_number]
    return render_to_response(template_name, {'item_list': items},
                              context_instance=RequestContext(request))


def item_template(request, template_name=DEFAULT_ITEM_TEMPLATE, item_id=None):
    from player.data.models import Item
    if item_id is None:
        raise Http404()
    item = Item.objects.get(pk=item_id)
    return render_to_response(template_name, {'item': item},
                              context_instance=RequestContext(request))


def views_to_register():
    """ returns views to be registered with dbresolver """
    return (
        {'view': render_template,
         'label': _('Generic view to render a template'),
         'params': (
             params.Template(name='template_name', label=_('Template to be rendered')),
         )},
        {'view': collection_template,
         'label': _('Generic view to render a collection'),
         'params': (
             params.Template(name='template_name', label=_('Template to be rendered'), default=DEFAULT_COLLECTION_TEMPLATE),
             CollectionParam(name='collection_slug', label=_('Collection to be rendered')),
             params.Integer(name='items_number', default=20, label=_('Number of items to be rendered')),
         )},
        {'view': item_template,
         'label': _('Generic view to render a item in a collection'),
         'params': (
             params.Template(name='template_name', label=_('Template to be rendered'), default=DEFAULT_ITEM_TEMPLATE),
         )},
    )



# -*- encoding: utf-8 -*-
# Django settings for example_project project.

from player.settings import *  # pyflakes:ignore

from os import path

ugettext = lambda s: s  # dummy ugettext function, as said on django docs

BASEDIR = path.dirname(path.abspath(__file__))

DEBUG = True
TEMPLATE_DEBUG = DEBUG

ADMINS = (
    # ('Your Name', 'your_email@example.com'),
)

MANAGERS = ADMINS

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',  # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': 'database.db',                   # Or path to database file if using sqlite3.
        'USER': '',                              # Not used with sqlite3.
        'PASSWORD': '',                          # Not used with sqlite3.
        'HOST': '',                              # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                              # Set to empty string for default. Not used with sqlite3.
    }
}

# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# On Unix systems, a value of None will cause Django to use the same
# timezone as the operating system.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = 'Europe/Madrid'

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = 'es'

LANGUAGES = (
    ('es', ugettext('Español')),
    ('en', ugettext('English')),
    ('fr', ugettext('Français')),
)

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = True

# If you set this to False, Django will not format dates, numbers and
# calendars according to the current locale
USE_L10N = True

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/home/media/media.lawrence.com/media/"
MEDIA_ROOT = path.join(BASEDIR, 'media')


# Absolute path to the directory static files should be collected to.
# Don't put anything in this directory yourself; store your static files
# in apps' "static/" subdirectories and in STATICFILES_DIRS.
# Example: "/home/media/media.lawrence.com/static/"
STATIC_ROOT = path.join(BASEDIR, 'static_root')

PROJECT_STATIC_ROOT = path.join(BASEDIR, 'static')

# Additional locations of static files
STATICFILES_DIRS = (
    # Put strings here, like "/home/html/static" or "C:/www/django/static".
    # Always use forward slashes, even on Windows.
    PROJECT_STATIC_ROOT,
)

# List of finder classes that know how to find static files in
# various locations.
STATICFILES_FINDERS = (
    'django.contrib.staticfiles.finders.FileSystemFinder',
    'django.contrib.staticfiles.finders.AppDirectoriesFinder',
#    'django.contrib.staticfiles.finders.DefaultStorageFinder',
)

# Make this unique, and don't share it with anybody.
SECRET_KEY = 'as!awi8ty*^0=im+%^6!m=q*xcjm@*lv2p5k0@#39-8y8vb0wf'

MIDDLEWARE_CLASSES += (
    # put here your project middlewares
    'staging.middleware.XsSharing',
)

ROOT_URLCONF = 'example_project.urls'

TEMPLATE_DIRS += (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    path.join(BASEDIR, 'templates'),
)

TEMPLATE_CONTEXT_PROCESSORS += (
    # put here your project context processors
)

INSTALLED_APPS += (
    # example apps
    'items',
    'news',
    'website',
)

INLINETRANS_MEDIA_URL = STATIC_URL + 'inlinetrans/'

# A sample logging configuration. The only tangible logging
# performed by this configuration is to send an email to
# the site admins on every HTTP 500 error.
# See http://docs.djangoproject.com/en/dev/topics/logging for
# more details on how to customize your logging configuration.
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'handlers': {
        'mail_admins': {
            'level': 'ERROR',
            'class': 'django.utils.log.AdminEmailHandler'
        }
    },
    'loggers': {
        'django.request': {
            'handlers': ['mail_admins'],
            'level': 'ERROR',
            'propagate': True,
        },
    }
}

# filebrowser settings
FILEBROWSER_MEDIA_ROOT = PROJECT_STATIC_ROOT
FILEBROWSER_DIRECTORY = ''
FILEBROWSER_MEDIA_URL = STATIC_URL

# django-playerlayer specific settings
CRAWLER_BACKEND = 'player.crawler.backends.mozenda'
CRAWLER_HOST = 'https://api.mozenda.com/rest'
CRAWLER_LOGIN = '9C4A9E46-9986-4A2D-8E28-5218F486AA40'
CRAWLER_PASSWORD = ''


# debug toolbar settings
def show_debug_toolbar(request):
    return request.user.is_staff

DEBUG_TOOLBAR_CONFIG = {
    'INTERCEPT_REDIRECTS': False,
    'SHOW_TOOLBAR_CALLBACK': show_debug_toolbar,
}
DEBUG_TOOLBAR_PANELS = (
    'debug_toolbar.panels.version.VersionDebugPanel',
    'debug_toolbar.panels.timer.TimerDebugPanel',
    'debug_toolbar.panels.settings_vars.SettingsVarsDebugPanel',
    'debug_toolbar.panels.headers.HeaderDebugPanel',
    'debug_toolbar.panels.request_vars.RequestVarsDebugPanel',
    #'debug_toolbar.panels.sql.SQLDebugPanel',
    'debug_toolbar.panels.template.TemplateDebugPanel',
    #'debug_toolbar.panels.cache.CacheDebugPanel',
    'debug_toolbar.panels.signals.SignalDebugPanel',
    'debug_toolbar.panels.logger.LoggingPanel',
)

# staging settings
STAGING_REPO_DIR = path.join(BASEDIR, 'repo')



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django import forms

from player.base.fields import ConfigFormField
from player.base.forms import BaseForm
from player.block import get_registered_blocks, get_block
from player.block.models import PlacedBlock


class PlacedBlockForm(BaseForm, forms.ModelForm):

    class Media:
        js = ('block/js/dynamic_config_params.js', )

    def __init__(self, *args, **kwargs):
        super(PlacedBlockForm, self).__init__(*args, **kwargs)
        instance = kwargs.pop('instance', None)
        block_choices = [("", "---------")]
        block_choices.extend([(block_key, '%s (%s)' % (unicode(block['label']), block_key)) for block_key, block in get_registered_blocks()])
        self.fields['block_path'] = forms.ChoiceField(choices=block_choices)
        if instance and 'config' in self.fields.keys():
            config = instance.get_block().get_config()
            if self.data:
                new_block_path = self.fields['block_path'].widget.value_from_datadict(self.data, self.files, self.add_prefix('block_path'))
                if instance.block_path != new_block_path:
                    # a uncommon situation if you changes the path selector dinamically when the block already exists
                    # we have to avoid a bad validation of config field because validate an old config_param definition
                    config = get_block(new_block_path).get_config()
            config_field = self.fields['config']
            config_field.set_config(config)

    class Meta:
        model = PlacedBlock


class AjaxPlacedBlockForm(PlacedBlockForm):

    config = ConfigFormField()

    class Meta:
        model = PlacedBlock
        fields = ('name', 'block_path', 'config')

    def save(self, *args, **kwargs):
        self.instance.placed_at = self.data.get('placed_at', None)
        blocks = PlacedBlock.objects.filter(placed_at=self.instance.placed_at).order_by('-order')
        if blocks:
            order = blocks[0].order + 1
        else:
            order = 0
        self.instance.order = order
        return super(AjaxPlacedBlockForm, self).save(*args, **kwargs)


class BlockConfigForm(forms.Form):
    config = ConfigFormField()



from django.contrib import admin

from player.block.forms import PlacedBlockForm
from player.block.models import PlacedBlock


class PlacedBlockAdmin(admin.ModelAdmin):
    form = PlacedBlockForm

    def get_form(self, request, obj=None, **kwargs):
        form = super(PlacedBlockAdmin, self).get_form(request, obj,
                                                      **kwargs)

        return form


admin.site.register(PlacedBlock, PlacedBlockAdmin)



"""
Serialize data to/from JSON
"""

from django.core.serializers.json import Serializer as JSONSerializer
from django.core.serializers.json import Deserializer as JSONDeserializer

from configfield.dbfields import JSONField


class Serializer(JSONSerializer):
    """
    Convert a queryset to JSON.
    """
    pass


def Deserializer(stream_or_string, **options):
    """
    Deserialize a stream or string of JSON data.
    Takes into account player.base.dbfields.JSONField issues
    """
    for obj in JSONDeserializer(stream_or_string, **options):
        for field in obj.object._meta.local_fields:
            if isinstance(field, JSONField):
                field_name = field.name
                setattr(obj.object, field_name, getattr(obj.object, 'get_%s_json' % field_name)())
        yield obj



from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _

from news.models import NewsItem


def news_list(request):
    all_news = NewsItem.objects.all()
    return render_to_response('news/news_list.html',
                              {'news_list': all_news},
                              context_instance=RequestContext(request))


def news_detail(request, news_slug):
    news_item = get_object_or_404(NewsItem, slug=news_slug)
    return render_to_response('news/news_detail.html',
                              {'news_item': news_item},
                              context_instance=RequestContext(request))


def views_to_register():
    """ returns views to be registered with dbresolver """
    return (
        (news_list, _('News listing')),
        (news_detail, _('News item detail')),
    )



from django.conf.urls.defaults import patterns, url

from news.views import news_list, news_detail


urlpatterns = patterns('',
    url(r'^$', news_list, name='news_index'),
    url(r'(?P<news_slug>[\w-]+)/', news_detail, name='news_detail'),
)



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from player.multimedia.datastructures import MediaDictionary


class MediaMiddleware(object):
    """
    This is a very simple middleware that parses a request
    and initializes the media bundle object in the current
    thread context.
    """

    def process_request(self, request):
        """ Initialize media contents for render_bundled_media and addmedia tags """
        request.media_holder = MediaDictionary()

    def process_response(self, request, response):
        """ Uninitialize media contents for render_bundled_media and addmedia tags """
        if hasattr(request, 'media_holder'):
            del request.media_holder
        return response



from django.conf.urls.defaults import patterns, url


urlpatterns = patterns('player.player_staging.views',
    url(r'^$', 'server_list', name='server_list'),
    url(r'^add/$', 'server_add', name='server_add'),
    url(r'^(?P<server_id>\d+)/$', 'server_detail', name='server_detail'),
    url(r'^(?P<server_id>\d+)/edit/$', 'server_edit', name='server_edit'),
    url(r'^publish/$', 'publish', name='publish'),
    url(r'^publish/new/$', 'publish_new', name='publish_new'),
    url(r'^publish/commit/$', 'commit', name='commit'),
    url(r'^create_repo_from_server/$', 'create_repo_from_server', name='create_repo_from_server'),
    url(r'^update_from_repository/$', 'update_from_repository', name='update_from_repository'),
    url(r'^log/$', 'log_view', name='log_view'),
    url(r'^log/(?P<changeset_id>[0-9a-f\:]+)/$', 'log_detail', name='log_detail'),
)



from django.conf import settings
from django.conf.urls.defaults import patterns, include, url
from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns

from dbresolver import autodiscover_views, get_dbresolver_patterns
from player.block import autodiscover_blocks

admin.autodiscover()
autodiscover_views()
autodiscover_blocks()

urlpatterns = patterns('',
    url(r'^$', 'website.views.home', name='home'),
    url(r'^news/', include('news.urls')),
    url(r'^manage/', include('player.manage.urls')),
    url(r'^admin/filebrowser/', include('filebrowser.urls')),
    url(r'^admin/doc/', include('django.contrib.admindocs.urls')),
    url(r'^admin/', include(admin.site.urls)),
    url(r'^staging/', include('staging.urls')),
    (r'^media/(.*)$', 'django.views.static.serve',
        {'document_root': settings.MEDIA_ROOT, 'show_indexes': True}),
)

urlpatterns += staticfiles_urlpatterns()
urlpatterns += get_dbresolver_patterns()



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-player.
#
# django-player is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-player is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-player.  If not, see <http://www.gnu.org/licenses/>.

import os
import shutil
import sys

from django.core.management.base import CommandError, _make_writeable


def copy_helper(style, name, directory):
    """
    Copies the project template into the specified directory.

    """
    # style -- A color style object (see django.core.management.color).
    # name -- The name of the project.
    # directory -- The directory to which the layout template should be copied.
    import re
    from player import settings as player_settings
    if not re.search(r'^[_a-zA-Z]\w*$', name):  # If it's not a valid directory name.
        # Provide a smart error message, depending on the error.
        if not re.search(r'^[_a-zA-Z]', name):
            message = 'make sure the name begins with a letter or underscore'
        else:
            message = 'use only numbers, letters and underscores'
        raise CommandError("%r is not a valid project name. Please %s." % (name, message))
    top_dir = os.path.join(directory, name)
    try:
        os.mkdir(top_dir)
    except OSError, e:
        raise CommandError(e)

    player_root = player_settings.PLAYERDIR

    skel_dir = os.path.join(player_root, 'skel', 'project')

    # Copy project template
    copy_dir(skel_dir, top_dir, name, False, style)


def remove_dst_if_exist(dst):
    if os.path.exists(dst):
        if os.path.islink(dst) or os.path.isfile(dst):
            os.remove(dst)
        else:
            shutil.rmtree(dst)


def copy_dir(source, dest, name, remove_if_exists, style, link=False):
    """ Copy directory recursively from a template directory """
    if remove_if_exists:
        remove_dst_if_exist(dest)
        os.makedirs(dest)

    for d, subdirs, files in os.walk(source):
        relative_dir = d[len(source) + 1:].replace('project_name', name)
        new_relative_dir = os.path.join(dest, relative_dir)
        if not os.path.exists(new_relative_dir):
            os.makedirs(new_relative_dir)
        for i, subdir in enumerate(subdirs):
            if subdir.startswith('.'):
                del subdirs[i]
        for f in files:
            if f.endswith('.pyc'):
                continue
            path_old = os.path.join(d, f)
            path_new = os.path.join(dest, relative_dir, f.replace('project_name', name))
            fp_old = open(path_old, 'r')
            fp_new = open(path_new, 'w')
            fp_new.write(fp_old.read().replace('{{ project_name }}', name))
            fp_old.close()
            fp_new.close()
            try:
                shutil.copymode(path_old, path_new)
                _make_writeable(path_new)
            except OSError:
                sys.stderr.write(style.NOTICE("Notice: Couldn't set permission bits on %s. You're probably using an uncommon filesystem setup. No problem.\n" % path_new))



# Copyright (c) 2010 by Manuel Saelices
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.context_processors import csrf
from django.core.exceptions import PermissionDenied
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseBadRequest, HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.simplejson import dumps
from django.utils.translation import ugettext as _

from player.block import get_block
from player.block.forms import BlockConfigForm, PlacedBlockForm, AjaxPlacedBlockForm
from player.block.models import PlacedBlock


@login_required
def block_list(request):
    """ List blocks in website """
    blocks = PlacedBlock.objects.all()
    return render_to_response('block/block_list.html',
                              {'block_list': blocks},
                              context_instance=RequestContext(request))


@login_required
def block_detail(request, block_id):
    """ View a template stored in database """
    block = PlacedBlock.objects.get(pk=block_id)
    return render_to_response('block/block_detail.html', {'placed_block': block},
                              context_instance=RequestContext(request))


@login_required
def block_add(request, block=None):
    """ Add a new block to place in website """
    if request.method == 'POST':
        form = PlacedBlockForm(request.POST, instance=block)
        if form.is_valid():
            block = form.save()
            msg = block and _(u'The block was successfully modified') or \
                  _(u'The block was successfully added')
            messages.success(request, msg)
            return HttpResponseRedirect(reverse('block_detail', args=(block.id, )))
    else:
        form = PlacedBlockForm(instance=block)
    return render_to_response(
        "block/block_edit.html",
        {'form': form, 'placed_block': block},
        context_instance=RequestContext(request),
    )


@login_required
def block_edit(request, block_id):
    """ Edit a placed block """
    block = get_object_or_404(PlacedBlock, pk=block_id)
    return block_add(request, block)


@login_required
def blocks_reorder(request):
    if not request.user.is_staff:
        return PermissionDenied()

    def relocate_blocks(items, cls):
        for order, item in enumerate(items):
            if "#" in item:
                item_split = item.split("#")
                element_id = int(item_split[0])
                placed_at = item_split[1]
                try:
                    cls.objects.get(id=element_id,
                                    placed_at=placed_at,
                                    order=order)
                except cls.DoesNotExist:
                    element = cls.objects.get(id=element_id)
                    element.order = order
                    element.placed_at = placed_at
                    element.save()

    mimetype = "application/json"
    if request.is_ajax() and request.POST and "new_order" in request.POST:
        new_order = request.POST['new_order']
        items = new_order.split(",")
        relocate_blocks(items, PlacedBlock)
        return HttpResponse(dumps(True), mimetype=mimetype)
    return HttpResponseBadRequest(mimetype=mimetype)


@login_required
def generate_blocks_configuration(request, block_id):
    if not request.user.is_staff:
        raise PermissionDenied()
    placed_block = PlacedBlock.objects.get(id=block_id)
    block = placed_block.get_block()
    config = block.get_config()
    if request.method == 'POST':
        form = BlockConfigForm(request.POST)
        form.fields['config'].set_config(config)
        if form.is_valid():
            placed_block.config = form.cleaned_data['config']
            placed_block.save()
            return HttpResponse('ok')
    else:
        form = BlockConfigForm()
        form.fields['config'].set_config(config)
    context = {
        'form': form,
        'placed_block': placed_block,
        'block': block,
    }
    context.update(csrf(request))
    return render_to_response('block/block_config.html', context)


@login_required
def generate_blocks_configuration_by_path(request, block_path):
    block = get_block(block_path)
    form = BlockConfigForm()
    config = block.get_config()
    form.fields['config'].set_config(config)
    context = {
        'form': form,
        'block': block,
    }
    context.update(csrf(request))
    return render_to_response('block/block_config_by_path.html', context)


@login_required
def get_block_config(request):
    block_path = request.GET.get('block_path', '')
    block = get_block(block_path)
    form = BlockConfigForm()
    config = block.get_config()
    form.fields['config'].set_config(config)
    context = {
            'form': form,
    }
    return render_to_response('block/block_ajax_edit_config.html', context)
    

@login_required
def ajax_block_add(request):
    """ Add a new block """
    if request.method == 'POST':
        form = AjaxPlacedBlockForm(request.POST)
        block = get_block(request.POST.get('block_path', ''))
        config = block.get_config()
        form.fields['config'].set_config(config)
        if form.is_valid():
            block = form.save()
            return HttpResponse(dumps({'html': block.render(request, {}), 'success': True}),
                                mimetype='text/plain')
    else:
        form = AjaxPlacedBlockForm()
    html = render_to_string("block/block_ajax_edit.html", {'form': form },
                            context_instance=RequestContext(request),
                           )
    return HttpResponse(dumps({'html': html, 'success': False}), mimetype='text/plain')



import urllib

from pygments import highlight
from pygments.lexers import DiffLexer
from pygments.formatters import HtmlFormatter

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _

from staging.models import StagingServer
from staging.utils import staging_manager
from player.player_staging.forms import StagingServerForm, CreateRepoFromServerForm


@login_required
def server_list(request):
    """ List Staging Servers stored in database """
    server_list = StagingServer.objects.all()
    return render_to_response('player_staging/server_list.html', {'server_list': server_list},
                              context_instance=RequestContext(request))


@login_required
def server_add(request, server=None):
    """ Add a new Staging Server to store in database """
    if request.method == 'POST':
        form = StagingServerForm(request.POST, instance=server)
        if form.is_valid():
            server = form.save()
            msg = server and _(u'The Staging Server was successfully modified') or \
                  _(u'The Staging Server was successfully added')
            messages.success(request, msg)
            return HttpResponseRedirect(reverse('server_detail', args=(server.id, )))
    else:
        form = StagingServerForm(instance=server)
    return render_to_response(
        "player_staging/server_edit.html",
        {'form': form, 'server': server},
        context_instance=RequestContext(request),
    )


@login_required
def server_detail(request, server_id):
    """ View Staging Server """
    server = StagingServer.objects.get(pk=server_id)
    return render_to_response('player_staging/server_detail.html', {'server': server},
                              context_instance=RequestContext(request))


@login_required
def server_edit(request, server_id):
    """ Edit a Staging Server """
    server = get_object_or_404(StagingServer, pk=server_id)
    return server_add(request, server)


@login_required
def create_repo_from_server(request):
    if request.method == 'POST':
        form = CreateRepoFromServerForm(request.POST)
        if form.is_valid():
            url_path = reverse('download_repository')
            url = urllib.urlopen('http://%s%s' % (form.cleaned_data['hostname'], url_path))
            staging_manager.unbundle(url)
            msg = _(u'The repository was successfully fetched')
            messages.success(request, msg)
            return HttpResponseRedirect(reverse('publish'))
    else:
        form = CreateRepoFromServerForm()
    return render_to_response(
        "player_staging/create_repo_from_server.html",
        {'form': form},
        context_instance=RequestContext(request),
    )


@login_required
def publish(request):
    """ Show current version """
    revision = staging_manager.get_revision()
    uninitialized = revision.split('\n')[0].replace('changeset:', '').strip().startswith('-1')
    return render_to_response('player_staging/publish.html', {'revision': revision, 'uninitialized': uninitialized},
                              context_instance=RequestContext(request))


@login_required
def publish_new(request):
    """ Creates the new version """
    staging_manager.staging_export()
    diff = staging_manager.get_diff()
    removed = staging_manager.get_removed_files()
    added = staging_manager.get_added_files()
    return render_to_response('player_staging/publish_new.html', {'added': added, 'removed': removed,
                              'plain_diff': diff,
                              'diff': highlight(diff, DiffLexer(), HtmlFormatter()),
                              'style': HtmlFormatter().get_style_defs('.highlight')},
                              context_instance=RequestContext(request))


@login_required
def commit(request):
    """ Comit changes into staging """
    staging_manager.commit()
    return HttpResponseRedirect(reverse('publish'))


@login_required
def log_view(request):
    """ Creates the new version """
    commits = staging_manager.get_logs()
    return render_to_response('player_staging/log_view.html', {'commits': commits},
                              context_instance=RequestContext(request))


@login_required
def log_detail(request, changeset_id):
    """ Creates the new version """
    log = staging_manager.get_log_for_revision(changeset_id)
    return render_to_response('player_staging/log_detail.html', {'log': log, 'changeset': changeset_id,
                              'diff': highlight(log['diff'], DiffLexer(), HtmlFormatter()),
                              'style': HtmlFormatter().get_style_defs('.highlight')},
                              context_instance=RequestContext(request))


@login_required
def update_from_repository(request):
    """ Comit changes into staging """
    staging_manager.staging_import()
    msg = _(u'The website was successfully updated from local repository')
    messages.success(request, msg)
    return HttpResponseRedirect(reverse('publish'))



from django.core.urlresolvers import reverse
from django.template import Library
from django.utils.translation import ugettext as _

register = Library()


@register.inclusion_tag('data/delete_collection_content.html', takes_context=True)
def content_delete(context, collection, next_url=None, link_title=None, title_message=None, confirmation_message=None, action=None):
    if not confirmation_message:
        confirmation_message = _('You can not undo this action. Are you sure?')
    else:
        confirmation_message = _(confirmation_message)
    if not link_title:
        link_title = _('Remove all the content')
    else:
        link_title = _(link_title)
    if not title_message:
        title_message = _('Remove elements from collection %s') % collection._meta.verbose_name
    else:
        title_message = _(title_message)
    if not action:
        action = reverse('content_delete')
    result = {'title_message': title_message,
              'confirmation_message': confirmation_message,
              'link_title': link_title,
              'next_url': next_url,
              'action': action,
             }
    app = collection._meta.app_label
    module = 'item'
    if context['request'].user.has_perm("%s.delete_%s" % (app, module)):
        result['collection_id'] = collection.id
    return result



# -*- coding: utf-8 -*-

import time
import socket
import SocketServer
import threading
from BaseHTTPServer import BaseHTTPRequestHandler, HTTPServer

from django.db import IntegrityError
from django.test import TestCase

from player.data.models import Collection, Item
from player.crawler import get_backend
from player.crawler.models import CrawlerCollection

TEST_BACKEND = 'player.crawler.backends.mozenda'


class FakeServer(SocketServer.ThreadingTCPServer):
    allow_reuse_address = 1    # Seems to make sense in testing environment

    def server_bind(self):
        """Override server_bind to store the server name."""
        SocketServer.TCPServer.server_bind(self)
        host, port = self.socket.getsockname()[:2]
        self.server_name = socket.getfqdn(host)
        self.server_port = port


class FakeHandler(BaseHTTPRequestHandler):

    def get_datas(self, path, band):
        # TODO: to implement
        return

    def do_GET(self):
        self.send_response(200)
        datas = self.get_datas(self.path, self.server.band)
        self.send_header('Content-type', 'text/xml; charset=utf-8')
        self.server.requests_processed.append(
            {'method': 'GET', 'path': self.path, 'data': datas})
        self.end_headers()
        self.wfile.write(datas)

    def do_POST(self):
        content_length = self.headers.getheader('content-length')
        self.rfile.read(int(content_length))

        self.send_response(200)
        self.end_headers()
        self.wfile.write('')


class MyThread(threading.Thread):

    def __init__(self, band, number_of_requests=1, port=8000):
        super(MyThread, self).__init__()
        self.number_of_requests = number_of_requests
        self.requests_processed = []
        self.band = band
        self.port = port

    def run(self):
        fake_server = HTTPServer(('', self.port), FakeHandler)
        fake_server.requests_processed = []
        fake_server.band = self.band
        while self.number_of_requests > 0:
            fake_server.handle_request()
            self.number_of_requests -= 1
            self.requests_processed = fake_server.requests_processed
            self.band = fake_server.band
        fake_server.socket.close()


# ./manage.py test data.DuplicateItemsTests --settings=settings_test
class DuplicateItemsTests(TestCase):

    def _fixture_setup(self):
        # we need to run the harvester server here because
        # the fixtures creates objects and the post-save
        # signals tries to connec to the harvester
        self.run_server(0, 4)
        super(DuplicateItemsTests, self)._fixture_setup()
        self.server.number_of_requests = 0
        self.server.join()

    def run_server(self, band, num_request):
        time.sleep(1)
        self.server = MyThread(band, num_request, 8000)
        self.server.start()
        time.sleep(2)

    def setUp(self):
        self.crawler_collection = CrawlerCollection.objects.get_or_create(collection_code='1014')[0]

        self.assertEquals(Item.objects.count(), 0)

        self.crawler_collection.check_deleted = True
        self.crawler_collection.new_elements_threshold = '0'
        self.crawler_collection.modified_elements_threshold = '0'
        self.crawler_collection.deleted_elements_threshold = '0'

        self.run_server(0, 3)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        self.number_of_requests = 0
        self.server.join()

    def test_add_item_fail(self):
        print '\ntest_add_item_fail'
        self.run_server(1, 6)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        self.server.number_of_requests = 0
        self.server.join()

        self.assertEquals(Item.objects.count(), 3)
        self.assertEquals(len(Item.objects.all()), 2)
        self.assertEquals(len(Item.objects.invalids()), 1)

        # Validamos el elemento:
        item_no_valid = Item.objects.invalids()[0]
        item_no_valid.is_valid = True
        item_no_valid.save()

        self.assertEquals(Item.objects.count(), 3)
        self.assertEquals(len(Item.objects.all()), 3)
        self.assertEquals(len(Item.objects.invalids()), 0)

        self.run_server(1, 3)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        self.server.number_of_requests = 0
        self.server.join()

        self.assertEquals(Item.objects.count(), 3)
        self.assertEquals(len(Item.objects.all()), 3)
        self.assertEquals(len(Item.objects.invalids()), 0)

    def test_modify_item_fail(self):
        print '\ntest_modify_item_fail'

        self.run_server(2, 6)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        self.server.number_of_requests = 0
        self.server.join()

        self.assertEquals(Item.objects.count(), 3)
        self.assertEquals(len(Item.objects.all()), 2)
        self.assertEquals(len(Item.objects.invalids()), 1)

        # validamos el elemento
        item_no_valid = Item.objects.invalids()[0]
        if len(Item.objects.filter(uid=item_no_valid.uid,
                                   collection=item_no_valid.collection,
                                   is_valid=True, is_deleted=False)):
            Item.objects.get(uid=item_no_valid.uid,
                             collection=item_no_valid.collection,
                             is_valid=True, is_deleted=False).delete()
        item_no_valid.is_valid = True
        item_no_valid.save()

        self.assertEquals(Item.objects.count(), 2)
        self.assertEquals(len(Item.objects.all()), 2)
        self.assertEquals(len(Item.objects.invalids()), 0)

        self.run_server(2, 3)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        self.server.number_of_requests = 0
        self.server.join()

        self.assertEquals(Item.objects.count(), 2)
        self.assertEquals(len(Item.objects.all()), 2)
        self.assertEquals(len(Item.objects.invalids()), 0)

    def test_remove_item_fail(self):
        print '\ntest_remove_item_fail'
        self.run_server(3, 6)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        self.server.number_of_requests = 0
        self.server.join()

        self.assertEquals(Item.objects.count(), 3)
        self.assertEquals(len(Item.objects.all()), 2)
        self.assertEquals(len(Item.objects.invalids()), 1)

        # validamos el elemento
        item_no_valid = Item.objects.invalids()[0]
        if not item_no_valid.content:
            Item.objects.get(uid=item_no_valid.uid,
                             collection=item_no_valid.collection,
                             is_valid=True, is_deleted=False).delete()
            item_no_valid.delete()

        self.assertTrue(item_no_valid.is_deleted)

        self.assertEquals(Item.objects.count(), 1)
        self.assertEquals(len(Item.objects.all()), 1)
        self.assertEquals(len(Item.objects.invalids()), 0)

        self.run_server(3, 3)
        get_backend(TEST_BACKEND).extract_collection_data(self.crawler_collection)
        self.server.number_of_requests = 0
        self.server.join()

        self.assertEquals(Item.objects.count(), 1)
        self.assertEquals(len(Item.objects.all()), 1)
        self.assertEquals(len(Item.objects.invalids()), 0)
        self.assertTrue(item_no_valid.is_deleted)


# ./manage.py test data.IntegrityTests
class IntegrityTests(TestCase):

    def test_integrity_item(self):
        """
        Test items integrity, defined in unique_together Meta option for Item model...
        """
        collection = Collection.objects.create(name="foo-collection")
        Item.objects.create(collection=collection, uid="foo", is_valid=True, is_deleted=True)
        Item.objects.create(collection=collection, uid="foo", is_valid=True, is_deleted=False)
        try:
            Item.objects.create(collection=collection, uid="foo", is_valid=True, is_deleted=False)
        except Exception, e:
            if isinstance(e, IntegrityError):
                print "Pass"
            else:
                print "Fail with %s" % type(e)



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

import time

from django import template
from django.core.cache import cache

from classytags.arguments import Argument
from classytags.core import Tag, Options
from classytags.parser import Parser
from compressor import CssCompressor, JsCompressor
from compressor.conf.settings import COMPRESS

register = template.Library()


MINT_DELAY = 30  # on how long any compression should take to be generated
REBUILD_TIMEOUT = 2592000  # rebuilds the cache every 30 days if nothing has changed


class MediaParser(Parser):

    def parse_blocks(self):
        super(MediaParser, self).parse_blocks()
        self.blocks['nodelist'] = self.parser.parse()


class RenderBundledMedia(Tag):
    name = 'render_bundled_media'

    options = Options(
        Argument('name'),
        parser_class=MediaParser,
    )

    def cache_get(self, key):
        packed_val = cache.get(key)
        if packed_val is None:
            return None
        val, refresh_time, refreshed = packed_val
        if (time.time() > refresh_time) and not refreshed:
            # Store the stale value while the cache
            # revalidates for another MINT_DELAY seconds.
            self.cache_set(key, val, timeout=MINT_DELAY, refreshed=True)
            return None
        return val

    def cache_set(self, key, val, timeout=REBUILD_TIMEOUT, refreshed=False):
        refresh_time = timeout + time.time()
        real_timeout = timeout + MINT_DELAY
        packed_val = (val, refresh_time, refreshed)
        return cache.set(key, packed_val, real_timeout)

    @property
    def nodelist(self):
        return self.blocks['nodelist']

    def render_tag(self, context, name, nodelist):
        request = context['request']
        rendered_contents = nodelist.render(context)
        content = request.media_holder[name].render()
        if COMPRESS:
            if name == 'css':
                compressor = CssCompressor(content)
            elif name == 'js':
                compressor = JsCompressor(content)
            output = self.cache_get(compressor.cachekey)
            if output is None:
                try:
                    output = compressor.output()
                    self.cache_set(compressor.cachekey, output)
                except:
                    from traceback import format_exc
                    raise Exception(format_exc())
        else:
            output = content  # no compression
        return '%s\n%s' % (output, rendered_contents)

register.tag(RenderBundledMedia)


class AddMediaParser(Parser):

    def parse_blocks(self):
        name = self.kwargs['name'].var.token
        self.blocks['nodelist'] = self.parser.parse(
            ('endaddmedia', 'endaddmedia %s' % name)
        )
        self.parser.delete_first_token()


class AddMedia(Tag):
    name = 'addmedia'
    options = Options(
        Argument('name'),
        parser_class=AddMediaParser,
    )

    def render_tag(self, context, name, nodelist):
        request = context['request']
        rendered_contents = nodelist.render(context)
        request.media_holder[name].append(rendered_contents)
        return ""

register.tag(AddMedia)



from django.conf.urls.defaults import patterns, url


urlpatterns = patterns('player.block.views',
    url(r'^$', 'block_list', name='block_list'),
    url(r'^add/$', 'block_add', name='block_add'),
    url(r'^(?P<block_id>\d+)/$', 'block_detail', name='block_detail'),
    url(r'^(?P<block_id>\d+)/edit/$', 'block_edit', name='block_edit'),
    url(r'ajax/order/', 'blocks_reorder', name='blocks_reorder'),
    url(r'ajax/config/(?P<block_id>\d+)$', 'generate_blocks_configuration',
        name='generate_blocks_configuration'),
    url(r'ajax/config_by_path/(?P<block_path>[\w.]+)$', 'generate_blocks_configuration_by_path',
        name='generate_blocks_configuration_by_path'),
    url(r'ajax/add/$', 'ajax_block_add', name='ajax_block_add'),
    url(r'ajax/add/config/$', 'get_block_config', name='get_block_config'),
)



from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext

from player.crawler.models import Report
from player.data.models import Collection


@login_required
def report_index(request):
    reports = Report.objects.all()
    return render_to_response(
        "crawler/report_index.html",
        {'reports': reports},
        context_instance=RequestContext(request),
    )


@login_required
def report_view(request, report_id):
    report = get_object_or_404(Report, pk=report_id)
    collections = Collection.objects.filter(crawler_collection=report.crawler_collection)
    return render_to_response(
        "crawler/report_view.html",
        {'report': report,
         'collections': collections,
         'added_items': report.added_items.all().order_by('collection'),
         'updated_items': report.updated_items.all().order_by('collection'),
        },
        context_instance=RequestContext(request),
    )


@login_required
def crawled_view(request, report_id):
    report = get_object_or_404(Report, pk=report_id)
    return render_to_response(
        "crawler/crawled_view.html",
        {'report': report},
        context_instance=RequestContext(request),
    )



from django.conf.urls.defaults import patterns, url


urlpatterns = patterns('player.crawler.views',
    url(r'^$', 'report_index', name='report_index'),
    url(r'^report/(?P<report_id>\w+)/$', 'report_view', name='report_view'),
    url(r'^report/(?P<report_id>\w+)/crawled_data/$', 'crawled_view', name='crawled_view'),
)



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django import template

register = template.Library()


@register.inclusion_tag('uitools/toolbar.html', takes_context=True)
def render_toolbar(context):
    return {
        'STATIC_URL': context['STATIC_URL'],
        'request': context['request'],
        'user': context['user'],
    }



import pprint as pprint_mod
from django.template import defaultfilters, Library
from django.utils import simplejson as json


register = Library()


@register.filter
def json_format(value):
    """
    Returns a pretty formatted json format
    """
    return json.dumps(value, indent=2)


@register.filter
def pprint(value, width=40):
    """
    Returns a pretty formatted python object
    """
    return pprint_mod.pformat(value, width=width)


@register.inclusion_tag('base/inc.head.html')
def head(heading, head_id=None, head_type='h3'):
    head_id = head_id or defaultfilters.slugify(heading)
    return {'heading': heading, 'head_id': head_id, 'head_type': head_type}



from django.contrib import admin

from player.logicaldelete import admin as logicaldelete_admin
from player.data.models import CollectionGroup, Collection, CollectionField, Item


class CollectionGroupAdmin(admin.ModelAdmin):
    pass


class ItemAdmin(logicaldelete_admin.ModelAdmin):
    list_display = ('uid', 'collection', 'is_deleted', )
    list_filter = ('collection', 'is_deleted', )
    change_form_template = 'admin/data/item_change_form.html'


class CollectionFieldInline(admin.TabularInline):
    model = CollectionField
    prepopulated_fields = {"slug": ("name", )}


class CollectionAdmin(admin.ModelAdmin):
    list_display = ('name', 'collectiongroup', )
    list_filter = ('collectiongroup', )
    inlines = (CollectionFieldInline, )


admin.site.register(CollectionGroup, CollectionGroupAdmin)
admin.site.register(Collection, CollectionAdmin)
admin.site.register(Item, ItemAdmin)



from os import path

PLAYERDIR = path.dirname(path.abspath(__file__))

MIDDLEWARE_CLASSES = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'pagination.middleware.PaginationMiddleware',
    'player.multimedia.middleware.MediaMiddleware',
    'player.block.middleware.RenderBlockMiddleware',
    'debug_toolbar.middleware.DebugToolbarMiddleware',
)

TEMPLATE_CONTEXT_PROCESSORS = (
    'django.contrib.auth.context_processors.auth',
    'django.core.context_processors.debug',
    'django.core.context_processors.i18n',
    'django.core.context_processors.media',
    'django.core.context_processors.static',
    'django.core.context_processors.request',
    'django.contrib.messages.context_processors.messages',
    'player.context_processors.collections',
)

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'dbtemplates.loader.Loader',
    'django.template.loaders.filesystem.Loader',
    'django.template.loaders.app_directories.Loader',
    'django.template.loaders.eggs.Loader',
)

TEMPLATE_DIRS = (
    path.join(PLAYERDIR, 'templates'),
)


INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.messages',
    'django.contrib.staticfiles',
    'django.contrib.admin',
    # required by django-playerlayer
    'south',
    'sorl.thumbnail',
    'dbtemplates',
    'pagination',
    'inplaceeditform',
    'inlinetrans',
    'johnny',
    'oembed',
    'cmsutils',
    'compressor',
    'announcements',
    'dbresolver',
    'filebrowser',
    'debug_toolbar',
    'staging',
    'configfield',
    # django-playerlayer
    'player.base',
    'player.block',
    'player.crawler',
    'player.data',
    'player.dbtemplate',
    'player.dburl',
    'player.logicaldelete',
    'player.multimedia',
    'player.player_staging',
    'player.manage',
)

INTERNAL_IPS = ('127.0.0.1', )

LOCALE_PATHS = (
    path.join(PLAYERDIR, 'locale'),
)

SLUG_RE = r'[-_\.\w]+'

LOGIN_URL = '/manage/login/'

LOGOUT_URL = '/manage/logout/'

MEDIA_URL = '/media/'

STATIC_URL = '/static/'

ADMIN_MEDIA_PREFIX = '/static/admin/'

# dbtemplates settings
DBTEMPLATES_USE_CODEMIRROR = True
DBTEMPLATES_MEDIA_PREFIX = STATIC_URL + 'codemirror/'
DBTEMPLATES_AUTO_POPULATE_CONTENT = False
DBTEMPLATES_CACHE_BACKEND = 'dbtemplates.cache.DjangoCacheBackend'

DEBUG_TOOLBAR_CONFIG = {
    'INTERCEPT_REDIRECTS': False,
}

DEBUG_TOOLBAR_PANELS = (
    'debug_toolbar.panels.version.VersionDebugPanel',
    'debug_toolbar.panels.timer.TimerDebugPanel',
    'debug_toolbar.panels.settings_vars.SettingsVarsDebugPanel',
    'debug_toolbar.panels.headers.HeaderDebugPanel',
    'debug_toolbar.panels.request_vars.RequestVarsDebugPanel',
    #'debug_toolbar.panels.sql.SQLDebugPanel',
    'debug_toolbar.panels.template.TemplateDebugPanel',
    #'debug_toolbar.panels.cache.CacheDebugPanel',
    'debug_toolbar.panels.signals.SignalDebugPanel',
    'debug_toolbar.panels.logger.LoggingPanel',
)

CRAWLER_BACKENDS = (
    ('player.crawler.backends.mozenda', 'Mozenda'),
    ('player.crawler.backends.feed', 'Feed'),
)

STAGING_MODELS_TO_SAVE = (
    ('block', 'PlacedBlock'),
    ('data', 'CollectionGroup'),
    ('data', 'Collection'),
    ('data', 'CollectionField'),
    ('crawler', 'CrawlerCollection'),
    ('dbresolver', 'URLPattern'),
)

SERIALIZATION_MODULES = {
    "json": "player.serializers.json",
}



# Copyright (c) 2010 by Manuel Saelices
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django import template
from django.conf import settings

from player.block.models import PlacedBlock


register = template.Library()


def _render_blocks(place, context):
    request = context['request']
    rendered_blocks = []
    active_blocks = PlacedBlock.objects.actives().order_by('order')

    for block in active_blocks:
        if not block.show(place, request, context):
            continue
        # append the block rendering to list
        rendered_blocks.append(block.render(request, context))

    wrapped_blocks = ['<div class="blockWrapper">%s</div>' % s for s in rendered_blocks]

    return "<div class='blockContainer'>%s" \
            "<input type=\"hidden\" class=\"blockPlace\" value=\"%s\">" \
            "</div>" \
            % ('\n'.join(wrapped_blocks), place)


class RenderBlocksNode(template.Node):

    def __init__(self, place):
        self.place = place

    def render(self, context):
        return _render_blocks(self.place, context)


@register.tag(name='render_blocks')
def do_render_blocks(parser, token):
    """
    Usage::
      {% render_blocks "leftsidebar" %}
    """
    bits = token.split_contents()
    tag_name = bits[0]
    if len(bits) != 2:
        raise template.TemplateSyntaxError('"%r" tag requires only two '
                                           'arguments' % tag_name)
    place = bits[1]
    if not (place[0] == place[-1] and place[0] in ('"', "'")):
        raise (template.TemplateSyntaxError, "%r tag's argument should be in "
                                             "quotes" % tag_name)
    return RenderBlocksNode(place[1:-1])


@register.inclusion_tag('block/header.html', takes_context=True)
def render_blocks_media(context):
    return {'request': context.get('request'),
            'STATIC_URL': settings.STATIC_URL,
            'content': context.get('content'),
            'user': context.get('user'),
            }

@register.inclusion_tag('block/block_toolbar.html', takes_context=True)
def block_toolbar(context, node_id):
    tag_context = {
        'request': context['request'],
    }
    if 'user' in context and context['user'].is_staff:
        tag_context.update({
            'is_staff': True,
            'node_id': node_id,
        })
    else:
        tag_context.update({
            'is_staff': False,
            'request': context['request'],
        })
    return tag_context



# Copyright (c) 2010 by Manuel Saelices
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.shortcuts import render_to_response, get_object_or_404
from django.template import RequestContext
from django.utils.translation import ugettext as _

from dbtemplates.models import Template

from player.dbtemplate.forms import TemplateAdminForm


@login_required
def template_list(request):
    """ List URLs patterns stored in database """
    templates = Template.objects.all()
    return render_to_response('dbtemplate/template_list.html',
                              {'template_list': templates},
                              context_instance=RequestContext(request))


@login_required
def template_detail(request, template_id):
    """ View a template stored in database """
    template = Template.objects.get(pk=template_id)
    return render_to_response('dbtemplate/template_detail.html', {'template': template},
                              context_instance=RequestContext(request))


@login_required
def template_add(request, template=None):
    """ Add a new template to store in database """
    if request.method == 'POST':
        form = TemplateAdminForm(request.POST, instance=template)
        if form.is_valid():
            template = form.save()
            msg = template and _(u'The template was successfully modified') or \
                  _(u'The template was successfully added')
            messages.success(request, msg)
            return HttpResponseRedirect(reverse('template_detail', args=(template.id, )))
    else:
        form = TemplateAdminForm(instance=template)
    return render_to_response(
        "dbtemplate/template_edit.html",
        {'form': form, 'template': template},
        context_instance=RequestContext(request),
    )


@login_required
def template_edit(request, template_id):
    """ Edit a template to store in database """
    template = get_object_or_404(Template, pk=template_id)
    return template_add(request, template)



import re
import sys
from copy import deepcopy
from cStringIO import StringIO

from django.db import IntegrityError
from django.utils.encoding import smart_str, smart_unicode

from player.crawler.models import Report
from player.data.models import Item


def collect_items_extracted(items_list, collection):
    """returns a boolean value that is True if the extraction is
    correct, and False in other case"""

    # first, checks if it's the first crawling (no items on the collection)
    success = False
    log = StringIO()
    added_items = []
    updated_items = []
    deleted_items_uid = []
    deleted_items = []
    if collection.item_set.count() == 0:  # first crawl, store all items
        uid_list = []
        indexes_to_remove = []
        for i, item in enumerate(items_list):
            if item.uid not in uid_list:
                item.save()
                uid_list.append(item.uid)
            else:
                indexes_to_remove.append(i)
        removed_items = 0
        for i in indexes_to_remove:
            # we remove in a second step in order to mantain the loop iteration's
            del items_list[i - removed_items]
            removed_items += 1
        success = True
        print >> log, u'First crawling'
        added_items = items_list
    else:  # some elements are not
        # get how many items are new, updated and deleted from the "old" items
        dict_new_items = {}
        for item in items_list:
            dict_new_items[item.uid] = item
        previous_items_uid = []
        new_items_uid = []

        # it needs separates the UIDs and make the checks about it
        # because on the extracted Items, the __unicode__ function
        # does not work for us, because uses the id value, and it's
        # not enough to identify a single item.
        for item in Item.objects.all().filter(collection=collection):  # valids
            previous_items_uid.append(item.uid)

        for item in items_list:
            new_items_uid.append(item.uid)

        previous_items_uid_set = set(previous_items_uid)
        new_items_uid_set = set(new_items_uid)
        created_items_uid = list(new_items_uid_set.difference(previous_items_uid_set))
        deleted_items_uid = collection.crawler_collection.check_deleted and list(previous_items_uid_set.difference(new_items_uid_set)) or []

        for deleted_item_uid in deleted_items_uid:
            try:
                item = Item.objects.get(collection=collection, uid=deleted_item_uid,
                                        is_deleted=False, is_valid=True)
                deleted_items.append({'uid': deleted_item_uid,
                                      'repr': unicode(item),
                                      'collection': unicode(item.collection)})
            except Item.DoesNotExist:  # if the item does not exists, don't mind
                pass
        updated_items_uid = list(new_items_uid_set.intersection(previous_items_uid_set))

        aux = [uid for uid in updated_items_uid]
        for uid in aux:
            new_content = dict_new_items[uid].content
            if not Item.objects.get(collection=collection, uid=uid,
                                    is_deleted=False,
                                    is_valid=True).has_changed(new_content):
                updated_items_uid.remove(uid)

        # for now only checks as numeric values
        new_elements_threshold = _convert_threshold(collection.crawler_collection.new_elements_threshold)
        modified_elements_threshold = _convert_threshold(collection.crawler_collection.modified_elements_threshold)
        deleted_elements_threshold = _convert_threshold(collection.crawler_collection.deleted_elements_threshold)

        if new_elements_threshold < len(created_items_uid) or \
           modified_elements_threshold < len(updated_items_uid) or \
           deleted_elements_threshold < len(deleted_items_uid):
            if new_elements_threshold < len(created_items_uid):
                response = u'Too many new elements (%s/%s) in collection %s' % (
                    len(created_items_uid),
                    new_elements_threshold,
                    collection,
                )
                print >> log, smart_str(response)
            if modified_elements_threshold < len(updated_items_uid):
                response = u'Too many modified elements (%s/%s) in collection %s' % (
                    len(updated_items_uid),
                    modified_elements_threshold,
                    collection,
                )
                print >> log, smart_str(response)
            if modified_elements_threshold < len(deleted_items_uid):
                response = u'Too many deleted elements (%s/%s) in collection %s' % (
                    len(deleted_items_uid),
                    deleted_elements_threshold,
                    collection,
                )
                print >> log, smart_str(response)
            success = False
        else:
            success = True
            print >> log, u'Updated elements'

        # create new elements
        for uid in created_items_uid:
            item = dict_new_items[uid]
            item.is_valid = success
            try:
                item.save()
            except IntegrityError:
                item = Item.objects.get(uid=item.uid,
                                        collection=item.collection,
                                        is_valid=success)
            added_items.append(item)

        # update new elements
        for uid in updated_items_uid:
            item = Item.objects.get(uid=uid, collection=collection,
                                    is_deleted=False, is_valid=True)
            new_content = dict_new_items[uid].content
            if success:
                item.update_content(new_content)
                item.save()
            else:
                try:
                    invalid_item = Item.objects.get(uid=uid, collection=collection,
                                                    is_deleted=False, is_valid=False)
                except Item.DoesNotExist:
                    invalid_item = deepcopy(item)
                    invalid_item.id = None
                    invalid_item.is_valid = False

                invalid_item.update_content(new_content)
                invalid_item.save()

            updated_items.append(item)

        # delete obsoletes elements (if proceed)
        if collection.crawler_collection.check_deleted:
            for uid in deleted_items_uid:
                item = Item.objects.get(uid=uid, collection=collection,
                                        is_deleted=False, is_valid=True)
                if success:
                    item.delete()
                else:
                    try:
                        invalid_item = Item.objects.get(uid=uid, collection=collection,
                                                        is_deleted=False, is_valid=False)
                    except Item.DoesNotExist:
                        invalid_item = deepcopy(item)
                        invalid_item.id = None
                        invalid_item.is_valid = False

                    invalid_item.content = u''
                    invalid_item.save()

    return success, log, added_items, updated_items, deleted_items_uid, deleted_items


def create_extraction_report(success, crawler_collection, items_dict, log, crawled_log=''):
    report = Report(
        crawler_collection=crawler_collection,
        crawling_time=crawler_collection.last_crawling_time,
    )
    report.deleted_items_json = items_dict['deleted']
    if success:
        report.return_code = 'suc'
        report.crawled_data = crawled_log
        report.traceback = ''
    else:
        report.return_code = 'err'
        report.traceback = u'%s\n%s' % (smart_unicode(log.getvalue()), crawled_log)
        report.crawled_data = ''
    report.save()
    # adding items to report
    for item in items_dict['added']:
        report.added_items.add(item)
    for item in items_dict['updated']:
        report.updated_items.add(item)


# ----- auxiliar functions -----


def _convert_threshold(value, total=None):
    """Returns an integer with the value. By now only checks
    integers, not percentages"""
    if not value:
        value = ''
    re_res = re.search('^\d+%$', value)
    if re_res != None:  # the expresion match, so remove the trailing '%'
        value = value[:-1]
    if value == '':
        return sys.maxint
    else:
        return int(value)



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

import copy

from django.template import loader, TemplateDoesNotExist
from django.utils.datastructures import SortedDict
from django.utils.encoding import smart_str


class NotProvidedMeta(type):

    def __nonzero__(self):
        # we ensure when some param have not been defined (get a NOT_PROVIDED value)
        # evaluate to False, to avoid some logic errors. See #1307 ticket.
        return False


class NOT_PROVIDED:
    __metaclass__ = NotProvidedMeta


class Param(object):
    """ Base class for all configuration parameters """

    def __init__(self, name=None, label=None, default=NOT_PROVIDED, choices=None):
        self.name = name
        self.label = label
        self.default = default
        self.choices = choices

    def __repr__(self):
        return smart_str(unicode(self))

    def __unicode__(self):
        return u"<%s, %s>" % (self.get_value(), self.get_type())

    def has_default(self):
        return self.default is not NOT_PROVIDED

    def get_type(self):
        return self.__class__.__name__

    def get_value(self):
        return getattr(self, 'value', self.default)

    def get_value_display(self):
        value = self.get_value()
        if value == NOT_PROVIDED:
            return None
        else:
            return value

    def get_value_from_datadict(self, data, name):
        return data.get(name)

    def is_valid(self, value):
        """ returns if value is valid in this kind of param,
            to implement form validation """
        return True

    def render(self, name, widget_attrs,
               template_name='configfields/param_widget.html',
               extra_context=None):
        # This method is to be implemented on children classes
        context = {'param': self, 'name': name, 'widget_attrs': widget_attrs}
        context.update(extra_context or {})
        return loader.render_to_string(template_name, context)


class Single(Param):
    pass


class Integer(Param):

    def get_parsed_value(self, value):
        if value is not None and isinstance(value, basestring) and value.strip('-').isdigit():
            try:
                return int(value)
            except ValueError:
                # return value if any error happends that will get validated
                pass
        return value

    def get_value(self):
        value = super(Integer, self).get_value()
        return self.get_parsed_value(value)

    def get_value_from_datadict(self, data, name):
        value = super(Integer, self).get_value_from_datadict(data, name)
        return self.get_parsed_value(value)

    def is_valid(self, value):
        return isinstance(value, int)


class PositiveInteger(Integer):

    def is_valid(self, value):
        return isinstance(value, int) and value >= 0


class Bool(Param):

    VAL_FALSE = ['false', 'none']

    def get_value(self):
        val = super(Bool, self).get_value()
        if not val or (getattr(val, 'lower', None) and val.lower() in self.VAL_FALSE):
            return False
        return bool(val)

    def get_value_from_datadict(self, data, name):
        val = super(Bool, self).get_value_from_datadict(data, name)
        if not val or (getattr(val, 'lower', None) and val.lower() in self.VAL_FALSE):
            return False
        return bool(val)

    def is_valid(self, value):
        return isinstance(value, bool)

    def render(self, name, widget_attrs,
               template_name='configfields/bool_widget.html',
               extra_context=None):
        return super(Bool, self).render(name, widget_attrs,
                                        template_name, extra_context)


class List(Param):

    def get_value_display(self):
        value = self.get_value()
        return u'\n'.join(value)

    def get_value_from_datadict(self, data, name):
        values = data.getlist(name)
        value = []
        for v in values:
            value += v.split('\r\n')
        # delete empty lines
        value = [v for v in value if v.strip()]
        return value

    def render(self, name, widget_attrs,
               template_name='configfields/list_widget.html',
               extra_context=None):
        return super(List, self).render(name, widget_attrs,
                                        template_name, extra_context)


class Text(Param):

    def render(self, name, widget_attrs,
               template_name='configfields/text_widget.html',
               extra_context=None):
        return super(Text, self).render(name, widget_attrs,
                                        template_name, extra_context)


class Template(Param):

    def is_valid(self, value):
        try:
            loader.get_template(value)
        except TemplateDoesNotExist:
            return False
        else:
            return True


# ----- configuration dictionary structure -----

class ConfigDict(SortedDict):

    def __init__(self, config_params, config_values):
        super(ConfigDict, self).__init__()
        for param in config_params:
            param = copy.copy(param)
            if config_values and param.name in config_values:
                param.value = config_values[param.name]
            self[param.name] = param

    def update(self, d):
        for k, v in d.iteritems():
            if v.get_value() != NOT_PROVIDED:
                self[k] = v



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext_lazy as _

from player.block import register_block
from player.data.blocks import TemplateCollectionBlock


class ItemListBlock(TemplateCollectionBlock):
    name = 'itemlistblock'
    label = _('List items from collection')

register_block(ItemListBlock)



from django.contrib import messages
from django.contrib.auth.decorators import login_required
from django.core.urlresolvers import reverse
from django.http import HttpResponse, HttpResponseRedirect, Http404
from django.template import RequestContext
from django.shortcuts import render_to_response, get_object_or_404
from django.utils.translation import ugettext
from django.utils import simplejson as json
from django.views.generic import list_detail

from player.crawler import get_backend
from player.crawler.models import Report
from player.crawler.exceptions import CrawlerHttpException
from player.data.forms import (CollectionGroupForm, CollectionForm,
                           ScrapAssociationWizard, CollectionCodeFormStep,
                           FieldMappingFormStep, ExtractionParamsFormStep,
                           DeleteItemsFormStep, SelectBackendFormStep)
from player.data.models import CollectionGroup, Item, Collection, CollectionField


# ----- collection group views -----

@login_required
def collectiongroup_list(request):
    return list_detail.object_list(
        request,
        queryset=CollectionGroup.objects.all(),
        template_name="data/collectiongroup_list.html",
        template_object_name="collectiongroup",
    )


@login_required
def collectiongroup_view(request, slug):
    collectiongroup = get_object_or_404(CollectionGroup, slug=slug)
    return render_to_response(
        "data/collectiongroup_view.html",
        {'collectiongroup': collectiongroup},
        context_instance=RequestContext(request),
    )


@login_required
def collectiongroup_add(request):
    if request.method == 'POST':
        form = CollectionGroupForm(request.POST)
        if form.is_valid():
            collectiongroup = form.save()
            messages.success(request, ugettext(u'The collectiongroup was successfully added'))
            return HttpResponseRedirect(reverse('collectiongroup_view', args=(collectiongroup.slug, )))
    else:
        form = CollectionGroupForm()
    return render_to_response(
        "data/collectiongroup_edit.html",
        {'form': form, 'collectiongroup': None},
        context_instance=RequestContext(request),
    )


@login_required
def collectiongroup_edit(request, slug):
    pass


# ----- Collection views -----


@login_required
def collection_add(request, collection=None):
    if request.method == 'POST':
        form = CollectionForm(request.POST, instance=collection)
        if form.is_valid():
            newcollection = form.save()
            form.field_formset.instance = newcollection
            form.field_formset.save()
            if collection:
                messages.success(request, ugettext(u'The collection was successfully modified'))
            else:
                messages.success(request, ugettext(u'The collection was successfully added'))
            return HttpResponseRedirect(reverse('collection_view', args=(newcollection.collectiongroup.slug, newcollection.slug, )))
    else:
        form = CollectionForm(instance=collection)
    return render_to_response(
        "data/collection_edit.html",
        {'form': form, 'collection': collection},
        context_instance=RequestContext(request),
    )


@login_required
def collection_list(request):
    return list_detail.object_list(
        request,
        queryset=Collection.objects.all().order_by('collectiongroup'),
        template_name="data/collection_list.html",
        template_object_name="collection",
    )


@login_required
def collection_view(request, slug, collection_slug):
    collection = get_object_or_404(Collection.objects.select_related(),
                                   slug=collection_slug)
    return render_to_response(
        "data/collection_view.html",
        {'collection': collection,
        },
        context_instance=RequestContext(request),
    )


@login_required
def collection_edit(request, slug, collection_slug):
    collection = get_object_or_404(Collection.objects.select_related(),
                                   slug=collection_slug)
    return collection_add(request, collection)


@login_required
def collection_edit_fields(request, slug, collection_slug):
    """ used for inplace edit in collection_view """
    # TODO: adapt to model changes in #42 ticket
    field_id = request.POST['id']
    value = request.POST['value']
    collection = get_object_or_404(Collection, slug=collection_slug)
    field = CollectionField.objects.get(collection=collection, slug=value)
    if field_id == 'repr-field':
        collection.repr_field = field
    collection.save()
    return HttpResponse(value)


@login_required
def collection_fields_in_json(request, slug, collection_slug):
    """ Returns JSON format of editable fields with inplace edit. Used in collection_view """
    field_id = request.GET['id']
    collection = get_object_or_404(Collection, slug=collection_slug)
    if field_id == 'repr-field':
        fields = [field.slug for field in collection.fields.all()]
    results = {}
    for field in fields:
        results[field] = field
    json_data = json.dumps(results)
    return HttpResponse(json_data, mimetype='application/json')


@login_required
def collection_publish(request, slug, collection_slug):
    collection = get_object_or_404(Collection, slug=collection_slug)
    collection.status = 'published'
    collection.save()
    collection.item_set.with_invalids().delete()
    messages.success(request, ugettext('Collection published successfully'))
    return HttpResponseRedirect(collection.get_absolute_url())


@login_required
def collection_unpublish(request, slug, collection_slug):
    collection = get_object_or_404(Collection, slug=collection_slug)
    collection.status = 'draft'
    collection.save()
    messages.success(request, ugettext('Collection unpublished successfully'))
    return HttpResponseRedirect(collection.get_absolute_url())


@login_required
def collection_crawl(request, slug, collection_slug):
    """ Launch a crawling job for a collection """
    collection = get_object_or_404(Collection.objects.select_related(), slug=collection_slug)
    if not collection.is_mapped():
        raise Http404('Collection %s is not mapped.' % collection)
    get_backend(collection.crawler_collection.backend).extract_collection_data(collection.crawler_collection)
    try:
        report = Report.objects.filter(crawler_collection=collection.crawler_collection).latest()
        code_str = report.get_return_code_display()
        if report.return_code == 'suc':
            inform = messages.success
        else:
            inform = messages.error
        message = u'<a href="%s">%s</a>' % (reverse('report_view', args=(report.id, )),
                                            ugettext('Collection crawled report: %(code)s') % {'code': code_str})
        inform(request, message)
    except Report.DoesNotExist:
        messages.error(request, ugettext('Collection crawled report not found. Probably an error has occurred'))
    except CrawlerHttpException, excp:
        messages.error(request,
                       ugettext('Crawler connection failed: [%(code)s]: %(error)s') %
                           {'code': excp.http_code, 'error': excp.error})
    return HttpResponseRedirect("%s#elements-tab" % collection.get_absolute_url())


@login_required
def collection_toggle_crawling_checked(request, slug, collection_slug):
    collection = get_object_or_404(Collection, slug=collection_slug)
    if request.method == "POST":
        collection.crawling_checked = not collection.crawling_checked
        collection.save()
    return HttpResponseRedirect(reverse("collection_view", None,
                                        kwargs={'slug': slug,
                                                'collection_slug': collection_slug}) + "#steps-tab")


@login_required
def scrap_association(request, slug, collection_slug):
    """ Map a collection with a crawler collection """
    collection = get_object_or_404(Collection.objects.select_related(), slug=collection_slug)
    if not bool(collection.fields.all()):
        raise Http404('Collection %s have no fields defined so it can not be mapped.' % collection)
    if not collection.crawler_collection:
        wizard = ScrapAssociationWizard((SelectBackendFormStep, CollectionCodeFormStep, FieldMappingFormStep, ExtractionParamsFormStep), collection=collection)
    else:
        wizard = ScrapAssociationWizard((SelectBackendFormStep, CollectionCodeFormStep, FieldMappingFormStep, ExtractionParamsFormStep, DeleteItemsFormStep), collection=collection)
    if collection.crawler_collection:
        collection_code = collection.crawler_collection.collection_code
    else:
        collection_code = None
    return wizard(request, extra_context={'collection_code': collection_code})


# ----- Item views -----


@login_required
def item_view(request, slug, item_id):
    item = get_object_or_404(Item.objects.with_deleted(), pk=item_id)
    if not item.is_valid:
        messages.error(request,
                       ugettext(u'This item is not valid.'))
    return render_to_response(
        "data/item_view.html",
        {'item': item},
        context_instance=RequestContext(request),
    )


@login_required
def item_validate(request, slug, item_id):
    item = get_object_or_404(Item.objects.with_deleted(), pk=item_id)
    if item.content:
        if len(Item.objects.filter(uid=item.uid, collection=item.collection,
                                   is_valid=True, is_deleted=False)):
            Item.objects.get(uid=item.uid, collection=item.collection,
                             is_valid=True, is_deleted=False).delete()
        item.is_valid = True
        item.save()
    else:
        Item.objects.get(uid=item.uid, collection=item.collection,
                         is_valid=True, is_deleted=False).delete()
        item.delete()
    messages.success(request, ugettext(u'This items has been validate'))

    return HttpResponseRedirect(reverse('player.data.views.item_view', args=(slug, item_id)))



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from configfield import params
from player.data.models import Collection


def get_collections_choices():
    return [(c.slug, c.name) for c in Collection.objects.all()]


class CollectionParam(params.Param):

    def __init__(self, *args, **kwargs):
        super(CollectionParam, self).__init__(*args, **kwargs)
        if 'choices' not in kwargs:
            self.choices = get_collections_choices

    def is_valid(self, value):
        return super(CollectionParam, self).is_valid(value) and \
               Collection.objects.filter(slug=value)

    def get_value(self):
        value = super(CollectionParam, self).get_value()
        try:
            return Collection.objects.get(slug=value)
        except Collection.DoesNotExist:
            return None



# Copyright (c) 2010 by Manuel Saelices
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django.conf import settings
from django.template import RequestContext
from django.template.loader import render_to_string
from django.utils.importlib import import_module
from django.utils.translation import ugettext_lazy as _

from configfield import params

_registry = {}


def register_block(block_class):
    block_key = '%s.%s' % (block_class.__module__, block_class.__name__)
    _registry[block_key] = {
        'label': block_class.label or block_class.name,
        'class': block_class,
    }


def get_registered_blocks():
    return _registry.items()


def get_block(block_path, placed_block=None):
    module, class_name = block_path.rsplit('.', 1)
    mod = import_module(module)
    block_class = getattr(mod, class_name)
    return block_class(placed_block)


class Block(object):
    name = 'not_defined'
    label = None
    config_params = ()  # configuration parameters, to be overriden

    def __init__(self, placed_block=None):
        self.placed_block = placed_block
        if placed_block is not None:
            self.place = placed_block.placed_at
        else:
            self.place = '_unplaced_'

    def update_context(self, context):
        pass  # to be overriden

    def render_block(self, request, template_name='block.html', block_title=None,
                     context=None):
        if context is None:
            context = {}
        block_context = {
            'block_title': block_title or self.placed_block.name,
            'place': self.place,
            'placed_block': self.placed_block,
            'block_name': self.name,
        }
        context.update(block_context)
        self.update_context(context)
        return render_to_string(template_name, context,
                                context_instance=RequestContext(request))

    def get_config(self):
        from configfield.params import ConfigDict
        return ConfigDict(self.config_params, getattr(self.placed_block, 'config', {}))

    def render(self, request, context):
        raise NotImplementedError()

    def show(self, place, request, context):
        return self.place == place


class TemplateBlock(Block):
    name = _('Template block')
    config_params = Block.config_params + (
        params.Template(
            name='template',
            label=_('Template to be used for rendering this block'),
        ),
    )

    def render(self, request, context):
        template_name = self.get_config()['template'].get_value()
        return self.render_block(request, template_name, context=context)

register_block(TemplateBlock)


def autodiscover_blocks():
    import imp

    for app in settings.INSTALLED_APPS:
        try:
            app_path = import_module(app).__path__
        except AttributeError:
            continue

        # use imp.find_module to find the app's forms.py
        try:
            imp.find_module('blocks', app_path)
        except ImportError:
            continue

        # import the app's blocks.py file
        blocks_module = import_module('%s.blocks' % app)



from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.template import Library
from django.utils.translation import ugettext as _

register = Library()


@register.inclusion_tag('manage/generic_delete.html', takes_context=True)
def generic_delete(context, content, next_url=None, link_title=None, title_message=None, confirmation_message=None, action=None):
    if not confirmation_message:
        confirmation_message = _('You can not undo this action. Are you sure?')
    else:
        confirmation_message = _(confirmation_message)
    if not link_title:
        link_title = _('Remove this item')
    else:
        link_title = _(link_title)
    if not title_message:
        title_message = _('Remove %(model)s') % {'model': content._meta.verbose_name}
    else:
        title_message = _(title_message)
    if not action:
        action = reverse('generic_delete')
    result = {'title_message': title_message,
              'confirmation_message': confirmation_message,
              'link_title': link_title,
              'next_url': next_url,
              'action': action,
             }
    app = content._meta.app_label
    module = content._meta.module_name
    if context['request'].user.has_perm("%s.delete_%s" % (app, module)):
        content_type = ContentType.objects.get(app_label=app, model=module)
        result['content_id'] = content.id
        result['content_type_id'] = content_type.id
    return result



from django import forms
from django.utils.translation import ugettext as _

from player.base.forms import BaseForm

from staging.models import StagingServer


class StagingServerForm(BaseForm, forms.ModelForm):

    class Meta:
        model = StagingServer


class CreateRepoFromServerForm(BaseForm, forms.Form):
    hostname = forms.CharField(widget=forms.RadioSelect)

    def __init__(self, *args, **kwargs):
        super(CreateRepoFromServerForm, self).__init__(*args, **kwargs)
        server_choices = []
        for server in StagingServer.objects.all():
            label = server.name
            is_ok, message = server.check_repository()
            if not is_ok:
                label += ". %s: %s" % (_('Be careful'), message)
            server_choices.append((server.hostname, label))
        self.fields['hostname'].widget.choices = server_choices



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime

from django.db import models
from django.db.models.signals import post_save
from django.utils.translation import ugettext
from django.utils.translation import ugettext_lazy as _

from configfield.dbfields import JSONField
from player.logicaldelete import models as logicaldelete_models
from player.base.dbfields import AutoSlugField
from player.crawler import get_backend
from player.crawler.models import CrawlerCollection
from player.data.exceptions import DateConversionError
from player.data.managers import ItemManager, CollectionFieldManager, CollectionManager
from player.data.structures import ItemData


STATUS_LIST = (
    ('draft', _('Draft')),
    ('published', _('Published')),
)

FIELD_TYPES = (
    ('int', _('Integer')),
    ('date', _('Date')),
    ('text', _('Text')),
    ('html', _('HTML text')),
    ('url', _('URL')),
)


class CollectionGroup(models.Model):
    """ Represent a entire web site to parse """
    name = models.CharField(_('name'), max_length=200, unique=True)
    slug = AutoSlugField(autofromfield='name')

    class Meta:
        verbose_name = _('CollectionGroup')

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('player.data.views.collectiongroup_view', [self.slug])


class Collection(models.Model):
    """ Represent a content type (news, events, etc) """
    name = models.CharField(_('Name'), max_length=200, unique=True)
    slug = models.SlugField()
    collectiongroup = models.ForeignKey(CollectionGroup, verbose_name=_('Group'),
                                        null=True, blank=True)
    crawler_collection = models.ForeignKey(CrawlerCollection, null=True, blank=True)
    status = models.CharField(max_length=100, choices=STATUS_LIST,
                              default='draft')
    repr_field = models.OneToOneField(
        'CollectionField', verbose_name=_('Field representing item'),
        related_name='repr_collection', null=True, blank=True,
        help_text=_('Field in collection schema that will represent to content'),
    )
    crawling_checked = models.BooleanField(_(u'Crawling checked'), default=False)
    objects = CollectionManager()
    last_crawling_time = models.DateTimeField(_('Last crawling time'),
                                              null=True, blank=True)

    class Meta:
        verbose_name = _('Collection')

    def __unicode__(self):
        return self.name

    @models.permalink
    def get_absolute_url(self):
        return ('player.data.views.collection_view', [self.collectiongroup.slug, self.slug])

    def _stripped_name(self):
        """ used to remove all '-' in slug for configuration file """
        return self.slug.replace('-', '')
    stripped_name = property(_stripped_name)

    def _collection_link(self):
        return get_backend(self.crawler_collection.backend).get_collection_link(self.crawler_collection)
    collection_link = property(_collection_link)

    def is_well_defined(self):
        mapped = True
        fields = CollectionField.objects.filter(collection=self)
        for field in fields:
            if field.mapped_field == None:
                mapped = False

        return self.repr_field and mapped

    def first_crawl_done(self):
        return (self.last_crawling_time != None)

    def is_published(self):
        return self.status == 'published'

    def is_publishable(self):
        """
        Returns if collection can be publishable as collection
        because it complains all the requirements.
        """
        return self.crawling_checked and \
               self.is_well_defined() and self.first_crawl_done()

    def is_mapped(self):
        """ Returns if collection has mapped any field to any crawling collection """
        return self.crawler_collection is not None and self.fields.is_mapped().exists()


class CollectionField(models.Model):
    name = models.CharField(_('Name'), max_length=200)
    slug = models.SlugField()
    field_type = models.CharField(_('Field type'), max_length=20,
                                  default='text', choices=FIELD_TYPES)
    collection = models.ForeignKey(Collection, related_name='fields')
    is_matchup = models.BooleanField(_('Is matchup'), default=False)
    custom_format = models.CharField(_('Custom format'), max_length=200,
                                    null=True, blank=True)
    mapped_field = models.CharField(_('Mapped field'), max_length=200,
                                    null=True, blank=True)

    objects = CollectionFieldManager()

    def __unicode__(self):
        return self.name

    def delete(self):
        collection = self.collection
        collection_modified = False
        if collection.repr_field == self:
            collection.repr_field = None
            collection.save()
        super(CollectionField, self).delete()


class Item(logicaldelete_models.Model):
    """ Represent a item (news item, event, etc.) """
    uid = models.CharField(max_length=200)
    content = JSONField(editable=False)
    collection = models.ForeignKey(Collection)
    title = models.CharField(max_length=300, default='')
    is_valid = models.BooleanField(_('Is valid'), default=True)

    objects = ItemManager()

    class Meta:
        verbose_name = _('Item')
        unique_together = (('uid', 'collection', 'is_valid', 'is_deleted'), )

    def __init__(self, *args, **kwargs):
        super(Item, self).__init__(*args, **kwargs)
        self.data = ItemData(self)

    def __unicode__(self):
        collection = self.collection
        repr_field = collection.repr_field
        if repr_field and self.content is not None and repr_field.slug in self.content:
            return self.get_value_from_field(repr_field)
        return self.get_uid()

    @models.permalink
    def get_manage_url(self):
        return ('player.data.views.item_view', [self.collection.collectiongroup.slug, self.id])

    def get_uid(self):
        if not self.id:
            return 'not-saved-%s' % self.uid
        return '%d-%s' % (self.id, self.uid)

    def get_title(self):
        return self.collection.repr_field and self.get_value_from_field(self.collection.repr_field) or self.uid

    def get_value_from_field(self, field):
        return self.get_value(field.slug)

    def get_value(self, field_slug):
        """ get item value using mapped schema """
        # TODO: do type conversion if needed
        value = self.content[field_slug]
        if value is None:
            return None  # avoid next SQL sentences
        field = self.collection.fields.get(slug=field_slug)
        if field.field_type == 'date':
            date_format = field.custom_format or '%d/%m/%Y'
            try:
                value = datetime.strptime(value, date_format).isoformat()
            except ValueError:
                raise DateConversionError(
                    ugettext(u'Date conversion error: "%(value)s" value is not in format "%(format)s"') %
                    {'value': value, 'format': date_format},
                )
        return value

    def update_content(self, new_content):
        """ Update content extracted from player.crawler """
        self.content = new_content
        self._update_parsed_data()

    def update_title(self):
        if self.collection.repr_field and self.collection.repr_field.slug in self.content:
            max_length = self._meta.get_field_by_name('title')[0].max_length
            slug = self.content[self.collection.repr_field.slug]
            self.title = slug and slug[:max_length] or u'No title'
        else:
            self.title = ''

    def save(self, *args, **kwargs):
        if not self.id and not self.content:  # new item
            self.content = {}
        self.update_title()
        super(Item, self).save(*args, **kwargs)

    def has_changed(self, new_content):
        return self.content != new_content


def handle_item_save(sender, instance, created, **kwargs):
    if created:
        Item.objects.only_deleted().filter(uid=instance.uid, collection=instance.collection).delete()

post_save.connect(handle_item_save, sender=Item)



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django.forms import ValidationError
from django.utils.translation import ugettext as _

from cmsutils.forms.fields import JSONFormField

from player.base.widgets import ConfigWidget


class ConfigFormField(JSONFormField):

    def __init__(self, show_debug=False, *args, **kwargs):
        super(ConfigFormField, self).__init__(*args, **kwargs)
        self.label = kwargs.get('label', _('Configuration'))
        self.widget = ConfigWidget(show_debug)
        self.config = {}

    def set_config(self, config):
        self.config = config
        self.widget.add_config_widgets(config)

    def clean(self, value):
        value = super(ConfigFormField, self).clean(value)
        for name, param in self.config.items():
            if not param.is_valid(value.get(name, None)):
                raise ValidationError(_('Error in "%(name)s" field') % {'name': param.label})
        return value



import datetime
import httplib2
import hashlib
import time

from lxml import etree

from django.conf import settings
from django.utils.encoding import smart_str

from player.crawler.models import CrawlerCollection, Report
from player.crawler.exceptions import CrawlerHttpException
from player.crawler.backends.utils import collect_items_extracted, create_extraction_report
from player.data.models import Collection, Item


def get_collection_link(crawler_collection):
    """
    This function will return a representative link to collection configured with
    the scrap tool.
    """
    return 'http://www.mozenda.com/console/#collections/%s/' % crawler_collection.collection_code


def get_collection_schema(collection_code):
    """
    This function will return the collection schema configured in your scrap tool
    with collection_code
    """
    params = [('Operation', 'Collection.GetFields'),
              ('CollectionID', collection_code)]
    response = _mozenda_request(params)

    if response[0]['status'] == '200':
        parser = etree.fromstring(response[1])
        fields = parser.findall('FieldList/Field')
        schema = {}
        for field in fields:
            field_name = field.find('Name').text
            field_id = field.find('FieldID').text
            field_is_system = field.find('IsSystem').text
            field_is_match_up = field.find('IsMatchUp').text
            schema[field_name] = {u'id': field_id,
                                  u'type': 'text',
                                  u'is_system': field_is_system,
                                  u'is_match_up': field_is_match_up}
        return schema


def load_collection_schema(crawler_collection):
    """
    This function will load the collection schema configured in your scrap tool
    with collection_code
    """
    schema = get_collection_schema(crawler_collection.collection_code)
    if schema:
        crawler_collection.schema = schema
        crawler_collection.save()


def has_new_data(crawler_collection):
    """ return True if scraping tool have new data to extract """
    collection_code = crawler_collection.collection_code
    last_job_time = crawler_collection.last_job_time
    if last_job_time is None:
        return True  # crawler never has extracted any data from mozenda
    mozenda_last_job_time = _get_last_job_time(collection_code)
    if mozenda_last_job_time is None:
        return False  # Mozenda never has launch any extraction Job
    if mozenda_last_job_time > last_job_time:
        return True  # Mozenda Job is newer than our data
    return False  # Mozenda Job is older


def extract_collection_data(crawler_collection):
    """
    This function will parse the collection as configured in your scrap tool.
    """
    collection_code = crawler_collection.collection_code
    # - Get view_id
    params = [('Operation', 'Collection.GetViews'),
              ('CollectionID', collection_code)]

    response = _mozenda_request(params)
    if response[0]['status'] == '200':
        parser = etree.fromstring(response[1])
        view_id = parser.find('ViewList/View/ViewID').text
    collections_to_extract = Collection.objects.filter(crawler_collection__collection_code=collection_code)

    params = [('Operation', 'View.GetItems'),
              ('ViewID', view_id)]

    response = _mozenda_request(params)
    if response[0]['status'] == '200':
        parser = etree.fromstring(response[1])
        nodes = parser.findall('ItemList/Item')
        items_dict = {'added': [], 'updated': [], 'deleted': []}
        for collection in collections_to_extract:
            items_extracted = []
            has_matchup_fields = collection.fields.is_mapped().filter(is_matchup=True).exists()
            for node in nodes:
                uid = hashlib.md5()
                field_dict = {}
                for field in collection.fields.all().is_mapped():
                    field_name = field.mapped_field
                    xml_field_name = field_name.replace(' ', '')
                    value = node.find(xml_field_name).text
                    if field.is_matchup or not has_matchup_fields:
                        uid.update(smart_str(value))
                    field_dict[field.slug] = value
                item = Item(uid=uid.hexdigest(), collection=collection)
                item.content = field_dict
                items_extracted.append(item)
                # note: the items are not save yet in database (only in memory)
                # when we confirm we passed the security thresholds, we will save them

            success, log, added_items, updated_items, deleted_items_uid, deleted_items = collect_items_extracted(items_extracted, collection)
            collection.last_crawling_time = datetime.datetime.now()
            collection.save()
            items_dict['added'] += added_items
            items_dict['updated'] += updated_items
            items_dict['deleted'] += deleted_items
        crawler_collection.last_crawling_time = datetime.datetime.now()
        crawler_collection.last_job_time = _get_last_job_time(collection_code)
        crawler_collection.save()
        affected_collections = Collection.objects.filter(crawler_collection=crawler_collection)

        create_extraction_report(success, crawler_collection, items_dict, log, response[1])
    else:  # if response status code is not 200
        report = Report(
            crawler_collection=crawler_collection,
            crawling_time=datetime.datetime.now(),
        )
        report.return_code = 'err'
        report.traceback = str(response)
        report.crawled_data = ''
        report.save()


def get_collection_codes(login, password=None):
    """returns a list of tuples (code, label)

    username is actually the API key for Mozenda. It's called
    username to keep persistence between Mozenda and Dapper backend"""

    params = [('Operation', 'Collection.GetList')]
    response = _mozenda_request(params)

    if response[0]['status'] == '200':
        parser = etree.fromstring(response[1])
        cols_ids = parser.findall('CollectionList/Collection/CollectionID')
        cols_names = parser.findall('CollectionList/Collection/Name')

        collections = []
        for i in range(len(cols_ids)):
            aux_collection = (cols_ids[i].text, cols_names[i].text)
            collections.append(aux_collection)

        return collections
    # TODO: if the response code is not good, raise an exception


# ----- testing functions (used in unit tests) -----


def create_test_crawler_collection():
    """ Create test collection """
    return CrawlerCollection.objects.create(
            collection_code=settings.CRAWLER_TEST_COLLECTION_CODE,
    )


def run_test_extraction_process():
    """ Run extraction of test data in scraping tool """
    params = [('Operation', 'Agent.Run'),
              ('AgentID', settings.CRAWLER_TEST_AGENT_ID)]
    response = _mozenda_request(params)
    if response[0]['status'] == '200':
        parser = etree.fromstring(response[1])
        job_id = parser.findall('JobID')[0].text
        job_status = 'Running'
        job_status_params = [('Operation', 'Job.Get'),
                             ('JobID', job_id)]
        while job_status != 'Done':
            job_status_response = _mozenda_request(job_status_params)
            if job_status_response[0]['status'] == '200':
                job_status_parser = etree.fromstring(job_status_response[1])
                job_status = job_status_parser.findall('Job/Status')[0].text
                time.sleep(2)


# ----- auxiliar functions -----


def _get_last_job_time(collection_code):
    params = [('Operation', 'Agent.GetJobs'),
              ('AgentID', collection_code)]
    response = _mozenda_request(params)
    if response[0]['status'] == '200':
        parser = etree.fromstring(response[1])
        for job_status in parser.findall('JobList/Job'):
            if job_status.find('Status').text == 'Done':
                end_date_str = job_status.find('Ended').text
                return datetime.datetime.strptime(end_date_str, r'%Y-%m-%d %H:%M:%S')
    return None


def _build_url_query(api_key, params=[]):
    """build the url query string to perform a query to mozenda"""
    ops = ''
    for i in params:
        ops += '&%s=%s' % (i[0], i[1])

    url = '%s?WebServiceKey=%s&Service=Mozenda10%s'
    return url % (settings.CRAWLER_HOST, api_key, ops)


def _mozenda_request(params):
    http = httplib2.Http()

    response = http.request(_build_url_query(settings.CRAWLER_LOGIN, params))
    if response[0]['status'] == '200':
        return response
    else:
        raise CrawlerHttpException(response[0]['status'],
                                   response[1])


def _get_collection_id_from_code(collection_code):
    """returns the collection id from the collection code"""
    collections = get_collection_codes(settings.CRAWLER_LOGIN)

    for col in collections:
        if col[1] == collection_code:
            return col[0]

    return ''  # TODO: probably is a better idea to raise an exception



from django.template import RequestContext
from django.shortcuts import render_to_response


def home(request):
    return render_to_response('website/home.html', {},
                              context_instance=RequestContext(request))



from django.conf.urls.defaults import patterns, url


urlpatterns = patterns('player.dbtemplate.views',
    url(r'^$', 'template_list', name='template_list'),
    url(r'^add/$', 'template_add', name='template_add'),
    url(r'^(?P<template_id>\d+)/$', 'template_detail', name='template_detail'),
    url(r'^(?P<template_id>\d+)/edit/$', 'template_edit', name='template_edit'),
)



from django.conf.urls.defaults import patterns, url


urlpatterns = patterns('player.dburl.views',
    url(r'^$', 'url_list', name='url_list'),
    url(r'^add/$', 'url_add', name='url_add'),
    url(r'^(?P<url_id>\d+)/$', 'url_detail', name='url_detail'),
    url(r'^(?P<url_id>\d+)/edit/$', 'url_edit', name='url_edit'),
)



from django.conf.urls.defaults import patterns, include, url


urlpatterns = patterns('',
    (r'^$', 'player.manage.views.index'),
    url(r'^generic_delete/$', 'player.manage.views.generic_delete', name='generic_delete'),
    url(r'^content_delete/$', 'player.manage.views.content_delete', name='content_delete'),
    (r'^block/', include('player.block.urls')),
    (r'^crawler/', include('player.crawler.urls')),
    (r'^data/', include('player.data.urls')),
    (r'^dbtemplates/', include('player.dbtemplate.urls')),
    (r'^dburls/', include('player.dburl.urls')),
    (r'^staging/', include('player.player_staging.urls')),
    url(r'^login/$', 'django.contrib.auth.views.login', name="auth_login"),
    url(r'^logout/$', 'django.contrib.auth.views.logout', name="auth_logout"),
    url(r'^inlinetrans/', include('inlinetrans.urls')),
)



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

# code taken from Merengue CMS

from django.db import models
from django.template import defaultfilters


def get_ordered_parents(model):
    """
    Returns a list of all the ancestor of this model as a list.
    """
    result = []
    for parent in model._meta.parents:
        result.append(parent)
        result.extend(get_ordered_parents(parent))
    return result


class AutoSlugField(models.SlugField):

    def __init__(self, autofromfield, *args, **kwargs):
        self.force_on_edit = kwargs.pop('force_on_edit', False)
        self.unique_on_parent_model = kwargs.pop('unique_on_parent_model', False)
        super(AutoSlugField, self).__init__(*args, **kwargs)
        self.autofromfield = autofromfield
        self.editable = kwargs.get('editable', False)

    def sluggify(self, name, objects):
        slug = defaultfilters.slugify(name)
        slug_num = slug
        n = 2
        filter_param = '%s__exact' % self.name
        filters = {filter_param: slug_num}
        while objects.filter(**filters):
            slug_num = slug + u'-%s' % n
            filters[filter_param] = slug_num
            n += 1
        return slug_num

    def _get_manager(self, instance):
        instance_model = instance.__class__
        if not self.unique_on_parent_model:
            return instance_model._default_manager
        models_to_iterate = [instance_model] + get_ordered_parents(instance_model)
        for model in models_to_iterate:
            if self in model._meta.local_fields:
                return model._default_manager

    def pre_save(self, instance, add):
        value = getattr(instance, self.autofromfield)

        objects_manager = self._get_manager(instance)

        if not instance.id:
            slug = self.sluggify(value, objects_manager.all())
        elif add or self.force_on_edit:
            slug = self.sluggify(value, objects_manager.exclude(id=instance.id))
        else:
            slug = getattr(instance, self.name)
        setattr(instance, self.name, slug)
        return slug

    def get_internal_type(self):
        return 'SlugField'



# Copyright (c) 2010 by Yaco Sistemas
#
# This file is part of django-playerlayer.
#
# django-playerlayer is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# django-playerlayer is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with django-playerlayer.  If not, see <http://www.gnu.org/licenses/>.

from django.utils.translation import ugettext_lazy as _

from configfield import params
from player.block import Block, register_block

from news.models import NewsItem


class NewsBlock(Block):
    name = 'newsblock'
    label = _('Latest news block')
    config_params = (
        params.PositiveInteger(
            name='limit',
            label=_('number of news for the "Latest news" block'),
            default=5,
        ),
    )

    def render(self, request, context):
        limit = self.get_config()['limit'].get_value()
        news_list = NewsItem.objects.all()[:limit]
        context.update({
            'news_list': news_list,
        })
        return self.render_block(
            request,
            template_name='news/news_block.html',
            context=context,
        )

register_block(NewsBlock)
